@@ -2,13 +2,14 @@
 #
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 #  SLEPc - Scalable Library for Eigenvalue Problem Computations
-#  Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+#  Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
 #
 #  This file is part of SLEPc.
 #  SLEPc is distributed under a 2-clause BSD license (see LICENSE).
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 #
 
+from __future__ import print_function
 import os, sys, time, shutil
 
 def AddDefine(conffile,name,value,prefix='SLEPC_'):
@@ -74,7 +75,7 @@ def WriteModulesFile(modules,version,sdir):
   modules.write('set slepc_dir %s\n' % sdir)
   modules.write('setenv SLEPC_DIR $slepc_dir\n')
 
-def WritePkgconfigFile(pkgconfig,version,pversion,sdir,isinstall,prefixdir,slflag):
+def WritePkgconfigFile(pkgconfig,version,pversion,sdir,isinstall,prefixdir,singlelib):
   ''' Write the contents of the pkg-config file '''
   pkgconfig.write('prefix=%s\n' % prefixdir)
   pkgconfig.write('exec_prefix=${prefix}\n')
@@ -88,9 +89,10 @@ def WritePkgconfigFile(pkgconfig,version,pversion,sdir,isinstall,prefixdir,slfla
   if not isinstall:
     pkgconfig.write(' -I'+os.path.join(sdir,'include'))
   pkgconfig.write('\nLibs:')
-  if slflag:
-    pkgconfig.write(' %s${libdir}' % slflag)
-  pkgconfig.write(' -L${libdir} -lslepc\n')
+  if singlelib:
+    pkgconfig.write(' -L${libdir} -lslepc\n')
+  else:
+    pkgconfig.write(' -L${libdir} -lslepcnep -lslepcpep -lslepcsvd -lslepceps -lslepcmfn -lslepclme -lslepcsys\n')
 
 def WriteCMakeConfigFile(cmakeconf):
   ''' Write the contents of the CMake configuration file '''
@@ -122,10 +124,10 @@ if 'LANG' in os.environ and os.environ['LANG'] != '' and os.environ['LANG'] != '
 
 # Check python version
 if not hasattr(sys, 'version_info') or not sys.version_info[0] == 2 or not sys.version_info[1] >= 6:
-  print '*******************************************************************************'
-  print '*       Python2 version 2.6 or higher is required to run ./configure          *'
-  print '*          Try: "python2.7 ./configure" or "python2.6 ./configure"            *'
-  print '*******************************************************************************'
+  print('*******************************************************************************')
+  print('*       Python2 version 2.6 or higher is required to run ./configure          *')
+  print('*          Try: "python2.7 ./configure" or "python2.6 ./configure"            *')
+  print('*******************************************************************************')
   sys.exit(4)
 
 # Set python path
@@ -160,15 +162,15 @@ checkpackages    = [arpack, blopex, blzpack, feast, primme, slicot, trlan, lapac
 
 # Print help if requested and check for wrong command-line options
 if argdb.PopHelp():
-  print 'SLEPc Configure Help'
-  print '-'*80
+  print('SLEPc Configure Help')
+  print('-'*80)
   for pkg in optionspackages:
     pkg.ShowHelp()
   sys.exit(0)
 argdb.ErrorIfNotEmpty()
 
 # Check enviroment and PETSc version
-print 'Checking environment...',
+print('Checking environment...', end=' ')
 petsc.InitDir(slepc.prefixdir)
 slepc.InitDir()
 petsc.LoadVersion()
@@ -180,13 +182,18 @@ if petsc.version < slepc.version:
 petsc.LoadConf()
 if not petsc.precision in ['double','single','__float128']:
   sys.exit('ERROR: This SLEPc version does not work with '+petsc.precision+' precision')
-if slepc.isinstall and not petsc.isinstall:
-  sys.exit('ERROR: SLEPc cannot be configured for non-source installation if PETSc is not configured in the same way.')
 
 # Check for empty PETSC_ARCH
 emptyarch = not ('PETSC_ARCH' in os.environ and os.environ['PETSC_ARCH'])
 if emptyarch:
-  archname = 'installed-'+petsc.arch
+  pseudoarch = 'arch-' + sys.platform.replace('cygwin','mswin')+ '-' + petsc.language
+  if petsc.debug:
+    pseudoarch += '-debug'
+  else:
+    pseudoarch += '-opt'
+  if not 'real' in petsc.scalar:
+    pseudoarch += '-' + petsc.scalar
+  archname = 'installed-'+pseudoarch
 else:
   archname = petsc.arch
 
@@ -204,7 +211,7 @@ log.write('Working directory: '+os.getcwd())
 log.write('Python version:\n'+sys.version)
 log.write('make: '+petsc.make)
 log.write('PETSc source directory: '+petsc.dir)
-log.write('PETSc install directory: '+petsc.destdir)
+log.write('PETSc install directory: '+petsc.prefixdir)
 log.write('PETSc version: '+petsc.lversion)
 if not emptyarch:
   log.write('PETSc architecture: '+petsc.arch)
@@ -287,10 +294,12 @@ if emptyarch:
 log.NewSection('Checking PETSc installation...')
 if petsc.version > slepc.version:
   log.Println('\nWARNING: PETSc version '+petsc.version+' is newer than SLEPc version '+slepc.version)
-if petsc.release != slepc.release:
-  log.Exit('ERROR: Cannot mix release and development versions of SLEPc and PETSc')
+if slepc.release=='1' and not petsc.release=='1':
+  log.Exit('ERROR: a release version of SLEPc requires a release version of PETSc, not a development version')
+if slepc.release=='0' and petsc.release=='1':
+  log.Exit('ERROR: a development version of SLEPc cannot be built with a release version of PETSc')
 if petsc.isinstall:
-  if os.path.realpath(petsc.destdir) != os.path.realpath(petsc.dir):
+  if os.path.realpath(petsc.prefixdir) != os.path.realpath(petsc.dir):
     log.Println('\nWARNING: PETSC_DIR does not point to PETSc installation path')
 petsc.Check()
 if not petsc.havepackage:
@@ -338,7 +347,7 @@ else:
 log.write('pkg-config file in '+pkgconfdir)
 slflag = ''
 if petsc.buildsharedlib: slflag = petsc.slflag
-WritePkgconfigFile(pkgconfig,slepc.lversion,petsc.version,slepc.dir,slepc.isinstall,slepc.prefixdir,slflag)
+WritePkgconfigFile(pkgconfig,slepc.lversion,petsc.version,slepc.dir,slepc.isinstall,slepc.prefixdir,petsc.singlelib)
 log.write('CMake configure file in '+confdir)
 WriteCMakeConfigFile(cmakeconf)
 
@@ -346,8 +355,8 @@ WriteCMakeConfigFile(cmakeconf)
 if not slepc.isinstall:
   WriteReconfigScript(reconfig,slepc.dir,argdb.UsedArgs())
   try:
-    os.chmod(reconfigpath,0775)
-  except OSError, e:
+    os.chmod(reconfigpath,0o775)
+  except OSError as e:
     log.Exit('ERROR: Unable to make reconfigure script executable:\n'+str(e))
 
 # Finish with configuration files (except slepcvars)
@@ -372,7 +381,7 @@ if slepc.isrepo and petsc.fortran:
       bfort = sowing.Install(archdir,petsc.make)
     log.NewSection('Generating Fortran stubs...')
     log.write('Using BFORT='+bfort)
-    sys.path.insert(0, os.path.abspath(os.path.join('bin','maint')))
+    sys.path.insert(0, os.path.abspath(os.path.join('lib','slepc','bin','maint')))
     import generatefortranstubs
     generatefortranstubs.main(slepc.dir,bfort,os.getcwd(),0)
     generatefortranstubs.processf90interfaces(slepc.dir,0)
@@ -395,17 +404,17 @@ if slepc.cmake:
   else:
     import cmakegen
     try:
-      cmakegen.main(slepc.dir,petsc.dir,petscdestdir=petsc.destdir)
-    except (OSError), e:
+      cmakegen.main(slepc.dir,petsc.dir,petscprefixdir=petsc.prefixdir)
+    except (OSError) as e:
       log.Exit('ERROR: Generating CMakeLists.txt failed:\n'+str(e))
     import cmakeboot
     try:
       cmakeok = cmakeboot.main(slepc.dir,petsc.dir,log=log)
-    except (OSError), e:
+    except (OSError) as e:
       log.Exit('ERROR: Booting CMake in PETSC_ARCH failed:\n'+str(e))
-    except (ImportError, KeyError), e:
+    except (ImportError, KeyError) as e:
       log.Exit('ERROR: Importing cmakeboot failed:\n'+str(e))
-    except (AttributeError), e:
+    except (AttributeError) as e:
       log.Println('\nxxx'+'='*73+'xxx')
       log.Println('WARNING: CMake builds are not available (initialization failed)')
       log.Println('You can ignore this warning (use default build), or try reconfiguring PETSc')
@@ -451,15 +460,15 @@ for pkg in checkpackages:
   pkg.ShowInfo()
 log.write('\nFinishing Configure Run at '+time.ctime(time.time()))
 log.write('='*79)
-print
-print 'xxx'+'='*73+'xxx'
+print()
+print('xxx'+'='*73+'xxx')
 if petsc.make_is_gnumake: buildtype = 'gnumake'
 elif cmakeok: buildtype = 'cmake'
 else: buildtype = 'legacy'
-print ' Configure stage complete. Now build the SLEPc library with ('+buildtype+' build):'
+print(' Configure stage complete. Now build the SLEPc library with ('+buildtype+' build):')
 if emptyarch:
-  print '   make SLEPC_DIR=$PWD PETSC_DIR='+petsc.dir
+  print('   make SLEPC_DIR=$PWD PETSC_DIR='+petsc.dir)
 else:
-  print '   make SLEPC_DIR=$PWD PETSC_DIR='+petsc.dir+' PETSC_ARCH='+archname
-print 'xxx'+'='*73+'xxx'
-print
+  print('   make SLEPC_DIR=$PWD PETSC_DIR='+petsc.dir+' PETSC_ARCH='+archname)
+print('xxx'+'='*73+'xxx')
+print()
