@@ -1,4 +1,5 @@
 #!/usr/bin/env python
+from __future__ import print_function
 import os, sys, shutil, commands
 
 try:
@@ -9,10 +10,10 @@ except NameError:
 class Installer:
   def __init__(self, args = None):
     if len(args)<6:
-      print '********************************************************************'
-      print 'Installation script error - not enough arguments:'
-      print './config/install.py SLEPC_DIR PETSC_DIR PETSC_ARCH DESTDIR LIB_SUFFIX RANLIB'
-      print '********************************************************************'
+      print('********************************************************************')
+      print('Installation script error - not enough arguments:')
+      print('./config/install.py SLEPC_DIR PETSC_DIR DESTDIR PETSC_ARCH LIB_SUFFIX RANLIB')
+      print('********************************************************************')
       sys.exit(1)
     self.rootDir     = args[0]
     self.petscDir    = args[1]
@@ -23,22 +24,38 @@ class Installer:
     self.copies = []
     return
 
+  def readDestDir(self, src):
+    try:
+      f = open(src)
+      for l in f.readlines():
+        r = l.split('=',1)
+        if len(r)!=2: continue
+        if r[0].strip() == 'SLEPC_DESTDIR':
+          break
+      f.close()
+    except:
+      print('********************************************************************')
+      print('Error reading SLEPC_DESTDIR from slepcvariables')
+      print('********************************************************************')
+      sys.exit(1)
+    return r[1].strip()
+
   def setupDirectories(self):
-    self.installDir        = self.destDir
     self.archDir           = os.path.join(self.rootDir, self.arch)
     self.rootIncludeDir    = os.path.join(self.rootDir, 'include')
     self.archIncludeDir    = os.path.join(self.rootDir, self.arch, 'include')
     self.rootConfDir       = os.path.join(self.rootDir, 'lib','slepc','conf')
     self.archConfDir       = os.path.join(self.rootDir, self.arch, 'lib','slepc','conf')
-    self.rootBinDir        = os.path.join(self.rootDir, 'bin')
+    self.rootBinDir        = os.path.join(self.rootDir, 'lib','slepc','bin')
     self.archBinDir        = os.path.join(self.rootDir, self.arch, 'bin')
     self.archLibDir        = os.path.join(self.rootDir, self.arch, 'lib')
     self.destIncludeDir    = os.path.join(self.destDir, 'include')
     self.destConfDir       = os.path.join(self.destDir, 'lib','slepc','conf')
     self.destLibDir        = os.path.join(self.destDir, 'lib')
-    self.destBinDir        = os.path.join(self.destDir, 'bin')
+    self.destBinDir        = os.path.join(self.destDir, 'lib','slepc','bin')
+    self.installDir        = self.readDestDir(os.path.join(self.archConfDir,'slepcvariables'))
     self.installIncludeDir = os.path.join(self.installDir, 'include')
-    self.installBinDir     = os.path.join(self.installDir, 'bin')
+    self.installBinDir     = os.path.join(self.installDir, 'lib','slepc','bin')
     self.rootShareDir      = os.path.join(self.rootDir, 'share')
     self.destShareDir      = os.path.join(self.destDir, 'share')
     return
@@ -46,24 +63,24 @@ class Installer:
   def checkDestdir(self):
     if os.path.exists(self.destDir):
       if os.path.samefile(self.destDir, self.rootDir):
-        print '********************************************************************'
-        print 'Incorrect prefix usage. Specified destDir same as current SLEPC_DIR'
-        print '********************************************************************'
+        print('********************************************************************')
+        print('Incorrect prefix usage. Specified destDir same as current SLEPC_DIR')
+        print('********************************************************************')
         sys.exit(1)
       if os.path.samefile(self.destDir, os.path.join(self.rootDir,self.arch)):
-        print '********************************************************************'
-        print 'Incorrect prefix usage. Specified destDir same as current SLEPC_DIR/PETSC_ARCH'
-        print '********************************************************************'
+        print('********************************************************************')
+        print('Incorrect prefix usage. Specified destDir same as current SLEPC_DIR/PETSC_ARCH')
+        print('********************************************************************')
         sys.exit(1)
       if not os.path.isdir(os.path.realpath(self.destDir)):
-        print '********************************************************************'
-        print 'Specified destDir', self.destDir, 'is not a directory. Cannot proceed!'
-        print '********************************************************************'
+        print('********************************************************************')
+        print('Specified destDir', self.destDir, 'is not a directory. Cannot proceed!')
+        print('********************************************************************')
         sys.exit(1)
       if not os.access(self.destDir, os.W_OK):
-        print '********************************************************************'
-        print 'Unable to write to ', self.destDir, 'Perhaps you need to do "sudo make install"'
-        print '********************************************************************'
+        print('********************************************************************')
+        print('Unable to write to ', self.destDir, 'Perhaps you need to do "sudo make install"')
+        print('********************************************************************')
         sys.exit(1)
     return
 
@@ -74,7 +91,7 @@ class Installer:
     if not os.path.exists(dst):
       os.makedirs(dst)
     elif not os.path.isdir(dst):
-      raise shutil.Error, 'Destination is not a directory'
+      raise shutil.Error('Destination is not a directory')
     srcname = src
     dstname = os.path.join(dst, os.path.basename(src))
     try:
@@ -84,12 +101,12 @@ class Installer:
       else:
         copyFunc(srcname, dstname)
         copies.append((srcname, dstname))
-    except (IOError, os.error), why:
+    except (IOError, os.error) as why:
       errors.append((srcname, dstname, str(why)))
-    except shutil.Error, err:
+    except shutil.Error as err:
       errors.extend((srcname,dstname,str(err.args[0])))
     if errors:
-      raise shutil.Error, errors
+      raise shutil.Error(errors)
     return copies
 
   def copyexamplefiles(self, src, dst, copyFunc = shutil.copy2):
@@ -119,7 +136,7 @@ class Installer:
     """Recursively copy the examples directories
     """
     if not os.path.isdir(dst):
-      raise shutil.Error, 'Destination is not a directory'
+      raise shutil.Error('Destination is not a directory')
 
     self.copyfile(os.path.join(src,'makefile'),dst)
     names  = os.listdir(src)
@@ -143,7 +160,7 @@ class Installer:
         nret2 = nret + nret2
     return nret2
 
-  def copytree(self, src, dst, symlinks = False, copyFunc = shutil.copy2, exclude = []):
+  def copytree(self, src, dst, symlinks = False, copyFunc = shutil.copy2, exclude = [], exclude_ext= ['.DSYM','.o','.pyc','.html'], recurse = 1):
     """Recursively copy a directory tree using copyFunc, which defaults to shutil.copy2().
 
        The copyFunc() you provide is only used on the top level, lower levels always use shutil.copy2
@@ -161,7 +178,7 @@ class Installer:
     if not os.path.exists(dst):
       os.makedirs(dst)
     elif not os.path.isdir(dst):
-      raise shutil.Error, 'Destination is not a directory'
+      raise shutil.Error('Destination is not a directory')
     errors = []
     for name in names:
       srcname = os.path.join(src, name)
@@ -170,28 +187,28 @@ class Installer:
         if symlinks and os.path.islink(srcname):
           linkto = os.readlink(srcname)
           os.symlink(linkto, dstname)
-        elif os.path.isdir(srcname):
-          copies.extend(self.copytree(srcname, dstname, symlinks,exclude = exclude))
-        elif not (os.path.basename(srcname) in exclude or os.path.splitext(os.path.basename(srcname))[1]=='.html'):
+        elif os.path.isdir(srcname) and recurse and not os.path.basename(srcname) in exclude:
+          copies.extend(self.copytree(srcname, dstname, symlinks,exclude = exclude, exclude_ext = exclude_ext))
+        elif os.path.isfile(srcname) and not os.path.basename(srcname) in exclude and os.path.splitext(name)[1] not in exclude_ext:
           copyFunc(srcname, dstname)
           copies.append((srcname, dstname))
         # XXX What about devices, sockets etc.?
-      except (IOError, os.error), why:
+      except (IOError, os.error) as why:
         errors.append((srcname, dstname, str(why)))
       # catch the Error from the recursive copytree so that we can
       # continue with other files
-      except shutil.Error, err:
+      except shutil.Error as err:
         errors.extend((srcname,dstname,str(err.args[0])))
     try:
       shutil.copystat(src, dst)
-    except OSError, e:
+    except OSError as e:
       if WindowsError is not None and isinstance(e, WindowsError):
         # Copying file access times may fail on Windows
         pass
       else:
         errors.extend((src, dst, str(e)))
     if errors:
-      raise shutil.Error, errors
+      raise shutil.Error(errors)
     return copies
 
 
@@ -206,6 +223,7 @@ class Installer:
       line = line.replace(os.path.realpath(os.path.join(self.rootDir, 'bin')), self.installBinDir)
       line = line.replace(os.path.join(self.rootDir, 'include'), self.installIncludeDir)
       line = line.replace(os.path.realpath(os.path.join(self.rootDir, 'include')), self.installIncludeDir)
+      line = line.replace(self.rootDir, self.installDir)
       # remove SLEPC_DIR/PETSC_ARCH variables from conf-makefiles. They are no longer necessary
       line = line.replace('${SLEPC_DIR}/${PETSC_ARCH}', self.installDir)
       line = line.replace('PETSC_ARCH=${PETSC_ARCH}', '')
@@ -222,6 +240,7 @@ class Installer:
     for file in ['slepc_rules', 'slepc_variables','slepcrules', 'slepcvariables']:
       self.fixConfFile(os.path.join(self.destConfDir,file))
     self.fixConfFile(os.path.join(self.destLibDir,'pkgconfig','SLEPc.pc'))
+    self.fixConfFile(os.path.join(self.destIncludeDir,'slepcconf.h'))
     return
 
   def createUninstaller(self):
@@ -253,7 +272,7 @@ for dir in dirs:
     pass
 ''')
     f.close()
-    os.chmod(uninstallscript,0744)
+    os.chmod(uninstallscript,0o744)
     return
 
   def installIncludes(self):
@@ -263,8 +282,8 @@ for dir in dirs:
     return
 
   def installConf(self):
-    self.copies.extend(self.copytree(self.rootConfDir, self.destConfDir, exclude = ['gmakegen.py','install.py']))
-    self.copies.extend(self.copytree(self.archConfDir, self.destConfDir))
+    self.copies.extend(self.copytree(self.rootConfDir, self.destConfDir, exclude = ['gmakegen.py','install.py','bfort-base.txt','bfort-mpi.txt','bfort-petsc.txt','bfort-slepc.txt']))
+    self.copies.extend(self.copytree(self.archConfDir, self.destConfDir, exclude = ['configure.log','error.log','files','gmake.log','make.log','test.log']))
     return
 
   def installBin(self):
@@ -309,27 +328,30 @@ for dir in dirs:
     return
 
   def installLib(self):
-    self.copies.extend(self.copytree(self.archLibDir, self.destLibDir, copyFunc = self.copyLib, exclude = ['.DIR']))
+    self.copies.extend(self.copytree(self.archLibDir, self.destLibDir, copyFunc = self.copyLib, exclude = ['.DIR'],recurse = 0))
+    self.copies.extend(self.copytree(os.path.join(self.archLibDir,'pkgconfig'), os.path.join(self.destLibDir,'pkgconfig'), copyFunc = self.copyLib, exclude = ['.DIR'],recurse = 0))
     return
 
 
   def outputInstallDone(self):
-    print '''\
+    arch=self.arch
+    if arch.startswith('installed-'): arch='""'
+    print('''\
 ====================================
 Install complete.
 Now to check if the libraries are working do (in current directory):
-make SLEPC_DIR=%s PETSC_DIR=%s PETSC_ARCH="" test
+make SLEPC_DIR=%s PETSC_DIR=%s PETSC_ARCH=%s check
 ====================================\
-''' % (self.installDir,self.petscDir)
+''' % (self.installDir,self.petscDir,arch))
     return
 
   def outputDestDirDone(self):
-    print '''\
+    print('''\
 ====================================
 Copy to DESTDIR %s is now complete.
 Before use - please copy/install over to specified prefix: %s
 ====================================\
-''' % (self.destDir,self.installDir)
+''' % (self.destDir,self.installDir))
     return
 
   def runsetup(self):
@@ -339,16 +361,16 @@ Before use - please copy/install over to specified prefix: %s
 
   def runcopy(self):
     if self.destDir == self.installDir:
-      print '*** Installing SLEPc at prefix location:',self.destDir, ' ***'
+      print('*** Installing SLEPc at prefix location:',self.destDir, ' ***')
     else:
-      print '*** Copying SLEPc to DESTDIR location:',self.destDir, ' ***'
+      print('*** Copying SLEPc to DESTDIR location:',self.destDir, ' ***')
     if not os.path.exists(self.destDir):
       try:
         os.makedirs(self.destDir)
       except:
-        print '********************************************************************'
-        print 'Unable to create', self.destDir, 'Perhaps you need to do "sudo make install"'
-        print '********************************************************************'
+        print('********************************************************************')
+        print('Unable to create', self.destDir, 'Perhaps you need to do "sudo make install"')
+        print('********************************************************************')
         sys.exit(1)
     self.installIncludes()
     self.installConf()
