@@ -1,7 +1,8 @@
+from __future__ import print_function
 #
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 #  SLEPc - Scalable Library for Eigenvalue Problem Computations
-#  Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+#  Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
 #
 #  This file is part of SLEPc.
 #  SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -20,15 +21,15 @@ class Log:
       self.filename = filename
 
   def Println(self,string):
-    print string
+    print(string)
     self.fd.write(string+'\n')
 
   def Print(self,string):
-    print string,
+    print(string, end=' ')
     self.fd.write(string+' ')
 
   def NewSection(self,string):
-    print 'done\n'+string,
+    print('done\n'+string, end=' ')
     sys.stdout.flush()
     self.fd.write('='*80+'\n'+string+'\n')
 
@@ -36,7 +37,7 @@ class Log:
     self.fd.write(string+'\n')
 
   def Exit(self,string):
-    print '\n'+string
+    print('\n'+string)
     if hasattr(self,'fd'):
       self.fd.write('\n'+string+'\n')
       self.fd.close()
