@@ -1,7 +1,7 @@
 #
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 #  SLEPc - Scalable Library for Eigenvalue Problem Computations
-#  Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+#  Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
 #
 #  This file is part of SLEPc.
 #  SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -65,6 +65,8 @@ class PETSc(package.Package):
     if 'PETSC_ARCH' in os.environ and os.environ['PETSC_ARCH']:
       self.isinstall = False
       self.arch = os.environ['PETSC_ARCH']
+      if os.path.basename(self.arch) != self.arch:
+        self.log.Exit(('ERROR: variable PETSC_ARCH must not be a full path\nYou set PETSC_ARCH=%s\nMaybe you meant PETSC_ARCH=%s'% (self.arch,os.path.basename(self.arch))))
       petscvariables = os.path.join(self.dir,self.arch,'lib','petsc','conf','petscvariables')
       petscconf_h = os.path.join(self.dir,self.arch,'include','petscconf.h')
     else:
@@ -90,8 +92,8 @@ class PETSc(package.Package):
           self.precision = v
         elif k == 'MAKE':
           self.make = v
-        elif k == 'DESTDIR':
-          self.destdir = v
+        elif k == 'PREFIXDIR':
+          self.prefixdir = v
         elif k == 'BFORT':
           self.bfort = v
         elif k == 'TEST_RUNS':
@@ -162,13 +164,3 @@ class PETSc(package.Package):
       else:
         self.log.Exit('ERROR: cannot process file ' + petscconf_h)
 
-    # empty PETSC_ARCH, guess an arch name
-    if self.isinstall and not self.arch:
-      self.arch = 'arch-' + sys.platform.replace('cygwin','mswin')+ '-' + self.language
-      if self.debug:
-        self.arch += '-debug'
-      else:
-        self.arch += '-opt'
-      if not 'real' in self.scalar:
-        self.arch += '-' + self.scalar
-
