@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -38,6 +38,7 @@ struct _BVOps {
   PetscErrorCode (*norm_end)(BV,PetscInt,NormType,PetscReal*);
   PetscErrorCode (*matmult)(BV,Mat,BV);
   PetscErrorCode (*copy)(BV,BV);
+  PetscErrorCode (*copycolumn)(BV,PetscInt,PetscInt);
   PetscErrorCode (*resize)(BV,PetscInt,PetscBool);
   PetscErrorCode (*getcolumn)(BV,PetscInt,Vec*);
   PetscErrorCode (*restorecolumn)(BV,PetscInt,Vec*);
@@ -45,7 +46,9 @@ struct _BVOps {
   PetscErrorCode (*restorearray)(BV,PetscScalar**);
   PetscErrorCode (*getarrayread)(BV,const PetscScalar**);
   PetscErrorCode (*restorearrayread)(BV,const PetscScalar**);
-  PetscErrorCode (*duplicate)(BV,BV*);
+  PetscErrorCode (*restoresplit)(BV,BV*,BV*);
+  PetscErrorCode (*gramschmidt)(BV,PetscInt,Vec,PetscBool*,PetscScalar*,PetscScalar*,PetscReal*,PetscReal*);
+  PetscErrorCode (*duplicate)(BV,BV);
   PetscErrorCode (*create)(BV);
   PetscErrorCode (*setfromoptions)(PetscOptionItems*,BV);
   PetscErrorCode (*view)(BV,PetscViewer);
@@ -68,10 +71,12 @@ struct _p_BV {
   Mat                matrix;       /* inner product matrix */
   PetscBool          indef;        /* matrix is indefinite */
   BVMatMultType      vmm;          /* version of matmult operation */
+  PetscBool          rrandom;      /* reproducible random vectors */
 
   /*---------------------- Cached data and workspace -------------------*/
-  Vec                Bx;           /* result of matrix times a vector x */
   Vec                buffer;       /* buffer vector used in orthogonalization */
+  Mat                Abuffer;      /* auxiliary seqdense matrix that wraps the buffer */
+  Vec                Bx;           /* result of matrix times a vector x */
   PetscObjectId      xid;          /* object id of vector x */
   PetscObjectState   xstate;       /* state of vector x */
   Vec                cv[2];        /* column vectors obtained with BVGetColumn() */
@@ -80,14 +85,17 @@ struct _p_BV {
   PetscObjectId      id[2];        /* object id of obtained vectors */
   PetscScalar        *h,*c;        /* orthogonalization coefficients */
   Vec                omega;        /* signature matrix values for indefinite case */
-  Mat                B,C;          /* auxiliary dense matrices for matmult operation */
-  PetscObjectId      Aid;          /* object id of matrix A of matmult operation */
   PetscBool          defersfo;     /* deferred call to setfromoptions */
   BV                 cached;       /* cached BV to store result of matrix times BV */
   PetscObjectState   bvstate;      /* state of BV when BVApplyMatrixBV() was called */
+  BV                 L,R;          /* BV objects obtained with BVGetSplit() */
+  PetscObjectState   lstate,rstate;/* state of L and R when BVGetSplit() was called */
+  PetscInt           lsplit;       /* the value of l when BVGetSplit() was called */
+  PetscInt           issplit;      /* >0 if this BV has been created by splitting (1=left, 2=right) */
+  BV                 splitparent;  /* my parent if I am a split BV */
   PetscRandom        rand;         /* random number generator */
-  PetscBool          rrandom;      /* reproducible random vectors */
   Mat                Acreate;      /* matrix given at BVCreateFromMat() */
+  Mat                Aget;         /* matrix returned for BVGetMat() */
   PetscBool          cuda;         /* true if GPU must be used in SVEC */
   PetscScalar        *work;
   PetscInt           lwork;
@@ -133,6 +141,10 @@ PETSC_STATIC_INLINE PetscErrorCode BV_IPMatMult(BV bv,Vec x)
 
   PetscFunctionBegin;
   if (((PetscObject)x)->id != bv->xid || ((PetscObject)x)->state != bv->xstate) {
+    if (!bv->Bx) {
+      ierr = MatCreateVecs(bv->matrix,&bv->Bx,NULL);CHKERRQ(ierr);
+      ierr = PetscLogObjectParent((PetscObject)bv,(PetscObject)bv->Bx);CHKERRQ(ierr);
+    }
     ierr = MatMult(bv->matrix,x,bv->Bx);CHKERRQ(ierr);
     bv->xid = ((PetscObject)x)->id;
     bv->xstate = ((PetscObject)x)->state;
@@ -201,36 +213,6 @@ PETSC_STATIC_INLINE PetscErrorCode BV_AllocateSignature(BV bv)
   PetscFunctionReturn(0);
 }
 
-/*
-  BV_AllocateMatMult - Allocate auxiliary matrices required for BVMatMult if not available.
-*/
-PETSC_STATIC_INLINE PetscErrorCode BV_AllocateMatMult(BV bv,Mat A,PetscInt m)
-{
-  PetscErrorCode ierr;
-  PetscObjectId  Aid;
-  PetscBool      create=PETSC_FALSE;
-  PetscInt       cols;
-
-  PetscFunctionBegin;
-  if (!bv->B) create=PETSC_TRUE;
-  else {
-    ierr = MatGetSize(bv->B,NULL,&cols);CHKERRQ(ierr);
-    ierr = PetscObjectGetId((PetscObject)A,&Aid);CHKERRQ(ierr);
-    if (cols!=m || bv->Aid!=Aid) {
-      ierr = MatDestroy(&bv->B);CHKERRQ(ierr);
-      ierr = MatDestroy(&bv->C);CHKERRQ(ierr);
-      create=PETSC_TRUE;
-    }
-  }
-  if (create) {
-    ierr = MatCreateDense(PetscObjectComm((PetscObject)bv),bv->n,PETSC_DECIDE,bv->N,m,NULL,&bv->B);CHKERRQ(ierr);
-    ierr = PetscLogObjectParent((PetscObject)bv,(PetscObject)bv->B);CHKERRQ(ierr);
-    ierr = MatAssemblyBegin(bv->B,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
-    ierr = MatAssemblyEnd(bv->B,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
-  }
-  PetscFunctionReturn(0);
-}
-
 /*
   BVAvailableVec: First (0) or second (1) vector available for
   getcolumn operation (or -1 if both vectors already fetched).
@@ -266,7 +248,15 @@ PETSC_INTERN PetscErrorCode BVDot_BLAS_Private(BV,PetscInt,PetscInt,PetscInt,Pet
 PETSC_INTERN PetscErrorCode BVDotVec_BLAS_Private(BV,PetscInt,PetscInt,const PetscScalar*,const PetscScalar*,PetscScalar*,PetscBool);
 PETSC_INTERN PetscErrorCode BVScale_BLAS_Private(BV,PetscInt,PetscScalar*,PetscScalar);
 PETSC_INTERN PetscErrorCode BVNorm_LAPACK_Private(BV,PetscInt,PetscInt,const PetscScalar*,NormType,PetscReal*,PetscBool);
-PETSC_INTERN PetscErrorCode BVOrthogonalize_LAPACK_Private(BV,PetscInt,PetscInt,PetscScalar*,PetscScalar*);
+PETSC_INTERN PetscErrorCode BVMatCholInv_LAPACK_Private(BV,Mat,Mat);
+PETSC_INTERN PetscErrorCode BVMatTriInv_LAPACK_Private(BV,Mat,Mat);
+PETSC_INTERN PetscErrorCode BVMatSVQB_LAPACK_Private(BV,Mat,Mat);
+PETSC_INTERN PetscErrorCode BVOrthogonalize_LAPACK_TSQR(BV,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscInt);
+PETSC_INTERN PetscErrorCode BVOrthogonalize_LAPACK_TSQR_OnlyR(BV,PetscInt,PetscInt,PetscScalar*,PetscScalar*,PetscInt);
+
+/* reduction operation used in BVOrthogonalize */
+PETSC_EXTERN MPI_Op MPIU_TSQR;
+PETSC_EXTERN void SlepcGivensPacked(void*,void*,PetscMPIInt*,MPI_Datatype*);
 
 #if defined(PETSC_HAVE_VECCUDA)
 #include <petsccuda.h>
@@ -319,4 +309,157 @@ PETSC_INTERN PetscErrorCode BV_StoreCoefficients_CUDA(BV,PetscInt,PetscScalar*,P
 
 #endif /* PETSC_HAVE_VECCUDA */
 
+/*
+   BV_CleanCoefficients_Default - Sets to zero all entries of column j of the bv buffer
+*/
+PETSC_STATIC_INLINE PetscErrorCode BV_CleanCoefficients_Default(BV bv,PetscInt j,PetscScalar *h)
+{
+  PetscErrorCode ierr;
+  PetscScalar    *hh=h,*a;
+  PetscInt       i;
+
+  PetscFunctionBegin;
+  if (!h) {
+    ierr = VecGetArray(bv->buffer,&a);CHKERRQ(ierr);
+    hh = a + j*(bv->nc+bv->m);
+  }
+  for (i=0;i<bv->nc+j;i++) hh[i] = 0.0;
+  if (!h) { ierr = VecRestoreArray(bv->buffer,&a);CHKERRQ(ierr); }
+  PetscFunctionReturn(0);
+}
+
+/*
+   BV_AddCoefficients_Default - Add the contents of the scratch (0-th column) of the bv buffer
+   into column j of the bv buffer
+*/
+PETSC_STATIC_INLINE PetscErrorCode BV_AddCoefficients_Default(BV bv,PetscInt j,PetscScalar *h,PetscScalar *c)
+{
+  PetscErrorCode ierr;
+  PetscScalar    *hh=h,*cc=c;
+  PetscInt       i;
+
+  PetscFunctionBegin;
+  if (!h) {
+    ierr = VecGetArray(bv->buffer,&cc);CHKERRQ(ierr);
+    hh = cc + j*(bv->nc+bv->m);
+  }
+  for (i=0;i<bv->nc+j;i++) hh[i] += cc[i];
+  if (!h) { ierr = VecRestoreArray(bv->buffer,&cc);CHKERRQ(ierr); }
+  PetscFunctionReturn(0);
+}
+
+/*
+   BV_SetValue_Default - Sets value in row j (counted after the constraints) of column k
+   of the coefficients array
+*/
+PETSC_STATIC_INLINE PetscErrorCode BV_SetValue_Default(BV bv,PetscInt j,PetscInt k,PetscScalar *h,PetscScalar value)
+{
+  PetscErrorCode ierr;
+  PetscScalar    *hh=h,*a;
+
+  PetscFunctionBegin;
+  if (!h) {
+    ierr = VecGetArray(bv->buffer,&a);CHKERRQ(ierr);
+    hh = a + k*(bv->nc+bv->m);
+  }
+  hh[bv->nc+j] = value;
+  if (!h) { ierr = VecRestoreArray(bv->buffer,&a);CHKERRQ(ierr); }
+  PetscFunctionReturn(0);
+}
+
+/*
+   BV_SquareSum_Default - Returns the value h'*h, where h represents the contents of the
+   coefficients array (up to position j)
+*/
+PETSC_STATIC_INLINE PetscErrorCode BV_SquareSum_Default(BV bv,PetscInt j,PetscScalar *h,PetscReal *sum)
+{
+  PetscErrorCode ierr;
+  PetscScalar    *hh=h;
+  PetscInt       i;
+
+  PetscFunctionBegin;
+  *sum = 0.0;
+  if (!h) { ierr = VecGetArray(bv->buffer,&hh);CHKERRQ(ierr); }
+  for (i=0;i<bv->nc+j;i++) *sum += PetscRealPart(hh[i]*PetscConj(hh[i]));
+  if (!h) { ierr = VecRestoreArray(bv->buffer,&hh);CHKERRQ(ierr); }
+  PetscFunctionReturn(0);
+}
+
+/*
+   BV_ApplySignature_Default - Computes the pointwise product h*omega, where h represents
+   the contents of the coefficients array (up to position j) and omega is the signature;
+   if inverse=TRUE then the operation is h/omega
+*/
+PETSC_STATIC_INLINE PetscErrorCode BV_ApplySignature_Default(BV bv,PetscInt j,PetscScalar *h,PetscBool inverse)
+{
+  PetscErrorCode    ierr;
+  PetscScalar       *hh=h;
+  PetscInt          i;
+  const PetscScalar *omega;
+
+  PetscFunctionBegin;
+  if (!(bv->nc+j)) PetscFunctionReturn(0);
+  if (!h) { ierr = VecGetArray(bv->buffer,&hh);CHKERRQ(ierr); }
+  ierr = VecGetArrayRead(bv->omega,&omega);CHKERRQ(ierr);
+  if (inverse) for (i=0;i<bv->nc+j;i++) hh[i] /= PetscRealPart(omega[i]);
+  else for (i=0;i<bv->nc+j;i++) hh[i] *= PetscRealPart(omega[i]);
+  ierr = VecRestoreArrayRead(bv->omega,&omega);CHKERRQ(ierr);
+  if (!h) { ierr = VecRestoreArray(bv->buffer,&hh);CHKERRQ(ierr); }
+  PetscFunctionReturn(0);
+}
+
+/*
+   BV_SquareRoot_Default - Returns the square root of position j (counted after the constraints)
+   of the coefficients array
+*/
+PETSC_STATIC_INLINE PetscErrorCode BV_SquareRoot_Default(BV bv,PetscInt j,PetscScalar *h,PetscReal *beta)
+{
+  PetscErrorCode ierr;
+  PetscScalar    *hh=h;
+
+  PetscFunctionBegin;
+  if (!h) { ierr = VecGetArray(bv->buffer,&hh);CHKERRQ(ierr); }
+  ierr = BV_SafeSqrt(bv,hh[bv->nc+j],beta);CHKERRQ(ierr);
+  if (!h) { ierr = VecRestoreArray(bv->buffer,&hh);CHKERRQ(ierr); }
+  PetscFunctionReturn(0);
+}
+
+/*
+   BV_StoreCoefficients_Default - Copy the contents of the coefficients array to an array dest
+   provided by the caller (only values from l to j are copied)
+*/
+PETSC_STATIC_INLINE PetscErrorCode BV_StoreCoefficients_Default(BV bv,PetscInt j,PetscScalar *h,PetscScalar *dest)
+{
+  PetscErrorCode ierr;
+  PetscScalar    *hh=h,*a;
+  PetscInt       i;
+
+  PetscFunctionBegin;
+  if (!h) {
+    ierr = VecGetArray(bv->buffer,&a);CHKERRQ(ierr);
+    hh = a + j*(bv->nc+bv->m);
+  }
+  for (i=bv->l;i<j;i++) dest[i-bv->l] = hh[bv->nc+i];
+  if (!h) { ierr = VecRestoreArray(bv->buffer,&a);CHKERRQ(ierr); }
+  PetscFunctionReturn(0);
+}
+
+#if defined(PETSC_HAVE_VECCUDA)
+#define BV_CleanCoefficients(a,b,c)   ((a)->cuda?BV_CleanCoefficients_CUDA:BV_CleanCoefficients_Default)((a),(b),(c))
+#define BV_AddCoefficients(a,b,c,d)   ((a)->cuda?BV_AddCoefficients_CUDA:BV_AddCoefficients_Default)((a),(b),(c),(d))
+#define BV_SetValue(a,b,c,d,e)        ((a)->cuda?BV_SetValue_CUDA:BV_SetValue_Default)((a),(b),(c),(d),(e))
+#define BV_SquareSum(a,b,c,d)         ((a)->cuda?BV_SquareSum_CUDA:BV_SquareSum_Default)((a),(b),(c),(d))
+#define BV_ApplySignature(a,b,c,d)    ((a)->cuda?BV_ApplySignature_CUDA:BV_ApplySignature_Default)((a),(b),(c),(d))
+#define BV_SquareRoot(a,b,c,d)        ((a)->cuda?BV_SquareRoot_CUDA:BV_SquareRoot_Default)((a),(b),(c),(d))
+#define BV_StoreCoefficients(a,b,c,d) ((a)->cuda?BV_StoreCoefficients_CUDA:BV_StoreCoefficients_Default)((a),(b),(c),(d))
+#else
+#define BV_CleanCoefficients(a,b,c)   BV_CleanCoefficients_Default((a),(b),(c))
+#define BV_AddCoefficients(a,b,c,d)   BV_AddCoefficients_Default((a),(b),(c),(d))
+#define BV_SetValue(a,b,c,d,e)        BV_SetValue_Default((a),(b),(c),(d),(e))
+#define BV_SquareSum(a,b,c,d)         BV_SquareSum_Default((a),(b),(c),(d))
+#define BV_ApplySignature(a,b,c,d)    BV_ApplySignature_Default((a),(b),(c),(d))
+#define BV_SquareRoot(a,b,c,d)        BV_SquareRoot_Default((a),(b),(c),(d))
+#define BV_StoreCoefficients(a,b,c,d) BV_StoreCoefficients_Default((a),(b),(c),(d))
+#endif /* PETSC_HAVE_VECCUDA */
+
 #endif
