@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -80,5 +80,11 @@ PETSC_DEPRECATED("Use MatCreateTile()") PETSC_STATIC_INLINE PetscErrorCode Slepc
 
 PETSC_EXTERN PetscBool SlepcInitializeCalled;
 
+#if defined(PETSC_USE_COMPLEX)
+#define SlepcLogFlopsComplex(a) PetscLogFlops((a))
+#else
+#define SlepcLogFlopsComplex(a) PetscLogFlops((4.0*a))
+#endif
+
 #endif
 
