@@ -3,18 +3,18 @@
 
 #define SLEPC_VERSION_RELEASE    1
 #define SLEPC_VERSION_MAJOR      3
-#define SLEPC_VERSION_MINOR      8
-#define SLEPC_VERSION_SUBMINOR   3
+#define SLEPC_VERSION_MINOR      9
+#define SLEPC_VERSION_SUBMINOR   2
 #define SLEPC_VERSION_PATCH      0
-#define SLEPC_RELEASE_DATE       "October 20, 2017"
-#define SLEPC_VERSION_DATE       "abr 03, 2018"
+#define SLEPC_RELEASE_DATE       "April 12, 2018"
+#define SLEPC_VERSION_DATE       "jul 02, 2018"
 
 #if !defined (SLEPC_VERSION_GIT)
-#define SLEPC_VERSION_GIT        "v3.8.3"
+#define SLEPC_VERSION_GIT        "v3.9.2"
 #endif
 
 #if !defined(SLEPC_VERSION_DATE_GIT)
-#define SLEPC_VERSION_DATE_GIT   "2018-04-03 16:53:45 +0200"
+#define SLEPC_VERSION_DATE_GIT   "2018-07-02 19:20:14 +0200"
 #endif
 
 #define SLEPC_VERSION_EQ(MAJOR,MINOR,SUBMINOR) \
