@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -28,7 +28,6 @@ static char help[] = "Illustrates the use of a region for filtering; the number
  */
 
 PetscErrorCode MatMult_Brussel(Mat,Vec,Vec);
-PetscErrorCode MatShift_Brussel(PetscScalar*,Mat);
 PetscErrorCode MatGetDiagonal_Brussel(Mat,Vec);
 PetscErrorCode MyStoppingTest(EPS,PetscInt,PetscInt,PetscInt,PetscInt,EPSConvergedReason*,void*);
 
@@ -116,7 +115,6 @@ int main(int argc,char **argv)
   */
   ierr = MatCreateShell(PETSC_COMM_WORLD,2*n,2*n,2*N,2*N,(void*)ctx,&A);CHKERRQ(ierr);
   ierr = MatShellSetOperation(A,MATOP_MULT,(void(*)())MatMult_Brussel);CHKERRQ(ierr);
-  ierr = MatShellSetOperation(A,MATOP_SHIFT,(void(*)())MatShift_Brussel);CHKERRQ(ierr);
   ierr = MatShellSetOperation(A,MATOP_GET_DIAGONAL,(void(*)())MatGetDiagonal_Brussel);CHKERRQ(ierr);
 
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -216,17 +214,6 @@ PetscErrorCode MatMult_Brussel(Mat A,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-PetscErrorCode MatShift_Brussel(PetscScalar* a,Mat Y)
-{
-  CTX_BRUSSEL    *ctx;
-  PetscErrorCode ierr;
-
-  PetscFunctionBeginUser;
-  ierr = MatShellGetContext(Y,(void**)&ctx);CHKERRQ(ierr);
-  ctx->sigma += *a;
-  PetscFunctionReturn(0);
-}
-
 PetscErrorCode MatGetDiagonal_Brussel(Mat A,Vec diag)
 {
   Vec            d1,d2;
