@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -130,7 +130,7 @@ PetscErrorCode EPSSetUp_KrylovSchur(EPS eps)
 {
   PetscErrorCode    ierr;
   PetscReal         eta;
-  PetscBool         isfilt;
+  PetscBool         isfilt=PETSC_FALSE;
   BVOrthogType      otype;
   BVOrthogBlockType obtype;
   EPS_KRYLOVSCHUR   *ctx = (EPS_KRYLOVSCHUR*)eps->data;
