@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -159,9 +159,9 @@ static PetscErrorCode EPSSliceGetEPS(EPS eps)
   BVOrthogType       orthog_type;
   BVOrthogRefineType orthog_ref;
   BVOrthogBlockType  ob_type;
-  Mat                A,B=NULL,Ar,Br=NULL;
+  Mat                A,B=NULL,Ar=NULL,Br=NULL;
   PetscInt           i;
-  PetscReal          h,a,b;
+  PetscReal          h,a,b,zero;
   PetscMPIInt        rank;
   EPS_SR             sr=ctx->sr;
   PC                 pc;
@@ -171,7 +171,7 @@ static PetscErrorCode EPSSliceGetEPS(EPS eps)
   STType             sttype;
   PetscObjectState   Astate,Bstate=0;
   PetscObjectId      Aid,Bid=0;
-  const MatSolverPackage stype;
+  MatSolverType      stype;
 
   PetscFunctionBegin;
   ierr = EPSGetOperators(eps,&A,&B);CHKERRQ(ierr);
@@ -258,12 +258,14 @@ static PetscErrorCode EPSSliceGetEPS(EPS eps)
   ierr = KSPGetType(ksp,&ksptype);CHKERRQ(ierr);
   ierr = KSPGetPC(ksp,&pc);CHKERRQ(ierr);
   ierr = PCGetType(pc,&pctype);CHKERRQ(ierr);
-  ierr = PCFactorGetMatSolverPackage(pc,&stype);CHKERRQ(ierr);
+  ierr = PCFactorGetMatSolverType(pc,&stype);CHKERRQ(ierr);
+  ierr = PCFactorGetZeroPivot(pc,&zero);CHKERRQ(ierr);
   ierr = STGetKSP(ctx->eps->st,&ksp);CHKERRQ(ierr);
   ierr = KSPSetType(ksp,ksptype);CHKERRQ(ierr);
   ierr = KSPGetPC(ksp,&pc);CHKERRQ(ierr);
   ierr = PCSetType(pc,pctype);CHKERRQ(ierr);
-  if (stype) { ierr = PCFactorSetMatSolverPackage(pc,stype);CHKERRQ(ierr); }
+  ierr = PCFactorSetZeroPivot(pc,zero);CHKERRQ(ierr);
+  if (stype) { ierr = PCFactorSetMatSolverType(pc,stype);CHKERRQ(ierr); }
 
   ierr = EPSSetConvergenceTest(ctx->eps,eps->conv);CHKERRQ(ierr);
   ierr = EPSSetInterval(ctx->eps,a,b);CHKERRQ(ierr);
