@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -48,6 +48,7 @@ typedef struct {
   void              *formFunctionBctx;
   PetscErrorCode    (*formFunctionA)(SNES,Vec,Vec,void*);
   void              *formFunctionActx;
+  PetscErrorCode    (*formFunctionAB)(SNES,Vec,Vec,Vec,void*);
 } EPS_POWER;
 
 PetscErrorCode EPSSetUp_Power(EPS eps)
@@ -104,7 +105,10 @@ PetscErrorCode EPSSetUp_Power(EPS eps)
     ierr = MatCreateVecs(A,&res,NULL);CHKERRQ(ierr);
     power->formFunctionA = formFunctionA;
     power->formFunctionActx = ctx;
-    if (power->update) { ierr = SNESSetFunction(power->snes,res,EPSPowerFormFunction_Update,ctx);CHKERRQ(ierr); }
+    if (power->update) {
+      ierr = SNESSetFunction(power->snes,res,EPSPowerFormFunction_Update,ctx);CHKERRQ(ierr);
+      ierr = PetscObjectQueryFunction((PetscObject)A,"formFunctionAB",&power->formFunctionAB);CHKERRQ(ierr);
+    }
     else { ierr = SNESSetFunction(power->snes,res,formFunctionA,ctx);CHKERRQ(ierr); }
     ierr = VecDestroy(&res);CHKERRQ(ierr);
 
@@ -226,15 +230,21 @@ static PetscErrorCode EPSPowerFormFunction_Update(SNES snes,Vec x,Vec y,void *ct
   EPS            eps;
   PetscReal      bx;
   Vec            Bx;
+  EPS_POWER      *power;
 
   PetscFunctionBegin;
   ierr = PetscObjectQuery((PetscObject)snes,"eps",(PetscObject *)&eps);CHKERRQ(ierr);
   if (!eps) SETERRQ(PETSC_COMM_WORLD,PETSC_ERR_ARG_NULL,"No composed EPS");
+  power = (EPS_POWER*)eps->data;
   Bx = eps->work[2];
-  ierr = EPSPowerUpdateFunctionB(eps,x,Bx);CHKERRQ(ierr);
+  if (power->formFunctionAB) {
+    ierr = (*power->formFunctionAB)(snes,x,y,Bx,ctx);CHKERRQ(ierr);
+  } else {
+    ierr = EPSPowerUpdateFunctionA(eps,x,y);CHKERRQ(ierr);
+    ierr = EPSPowerUpdateFunctionB(eps,x,Bx);CHKERRQ(ierr);
+  }
   ierr = VecNorm(Bx,NORM_2,&bx);CHKERRQ(ierr);
   ierr = Normalize(Bx,bx,NULL);CHKERRQ(ierr);
-  ierr = EPSPowerUpdateFunctionA(eps,x,y);CHKERRQ(ierr);
   ierr = VecAXPY(y,-1.0,Bx);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -332,12 +342,13 @@ PetscErrorCode EPSSolve_Power(EPS eps)
   else Bx = NULL;
 
   if (power->shift_type != EPS_POWER_SHIFT_CONSTANT) { ierr = STGetKSP(eps->st,&ksp);CHKERRQ(ierr); }
-  if (eps->useds) { ierr = DSGetLeadingDimension(eps->ds,&ld);CHKERRQ(ierr); }
   if (power->nonlinear) {
     ierr = PetscObjectCompose((PetscObject)power->snes,"eps",(PetscObject)eps);CHKERRQ(ierr);
     if (power->update) {
       ierr = EPSPowerComputeInitialGuess_Update(eps);CHKERRQ(ierr);
     }
+  } else {
+    ierr = DSGetLeadingDimension(eps->ds,&ld);CHKERRQ(ierr);
   }
   if (!power->update) {
     ierr = EPSGetStartVector(eps,0,NULL);CHKERRQ(ierr);
@@ -855,9 +866,9 @@ static PetscErrorCode EPSPowerGetSNES_Power(EPS eps,SNES *snes)
   PetscFunctionBegin;
   if (!power->snes) {
     ierr = SNESCreate(PetscObjectComm((PetscObject)eps),&power->snes);CHKERRQ(ierr);
+    ierr = PetscObjectIncrementTabLevel((PetscObject)power->snes,(PetscObject)eps,1);CHKERRQ(ierr);
     ierr = SNESSetOptionsPrefix(power->snes,((PetscObject)eps)->prefix);CHKERRQ(ierr);
     ierr = SNESAppendOptionsPrefix(power->snes,"eps_power_");CHKERRQ(ierr);
-    ierr = PetscObjectIncrementTabLevel((PetscObject)power->snes,(PetscObject)eps,1);CHKERRQ(ierr);
     ierr = PetscLogObjectParent((PetscObject)eps,(PetscObject)power->snes);CHKERRQ(ierr);
   }
   *snes = power->snes;
@@ -937,9 +948,7 @@ PetscErrorCode EPSView_Power(EPS eps,PetscViewer viewer)
         ierr = PetscViewerASCIIPrintf(viewer,"  updating the residual monolithically\n");CHKERRQ(ierr);
       }
       if (!power->snes) { ierr = EPSPowerGetSNES(eps,&power->snes);CHKERRQ(ierr); }
-      ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
       ierr = SNESView(power->snes,viewer);CHKERRQ(ierr);
-      ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
     } else {
       ierr = PetscViewerASCIIPrintf(viewer,"  %s shifts\n",EPSPowerShiftTypes[power->shift_type]);CHKERRQ(ierr);
     }
