@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -42,11 +42,11 @@ typedef struct {
   PetscInt     L_max;      /* maximum number of columns of the source matrix V */
   PetscReal    spurious_threshold; /* discard spurious eigenpairs */
   PetscBool    isreal;     /* T(z) is real for real z */
+  PetscInt     npart;      /* number of partitions */
   PetscInt     refine_inner;
   PetscInt     refine_blocksize;
   /* private data */
   PetscReal    *sigma;     /* threshold for numerical rank */
-  PetscInt     num_subcomm;
   PetscInt     subcomm_id;
   PetscInt     num_solve_point;
   PetscScalar  *weight;
@@ -55,30 +55,73 @@ typedef struct {
   BV           V;
   BV           S;
   BV           Y;
-  KSP          *ksp;
+  KSP          *ksp;       /* ksp array for storing factorizations at integration points */
   PetscBool    useconj;
   PetscReal    est_eig;
   PetscSubcomm subcomm;
-  PetscBool    usest;
 } NEP_CISS;
 
-static PetscErrorCode SetSolverComm(NEP nep)
+/* destroy KSP objects when the number of solve points changes */
+PETSC_STATIC_INLINE PetscErrorCode NEPCISSResetSolvers(NEP nep)
+{
+  PetscErrorCode ierr;
+  PetscInt       i;
+  NEP_CISS       *ctx = (NEP_CISS*)nep->data;
+
+  PetscFunctionBegin;
+  if (ctx->ksp) {
+    for (i=0;i<ctx->num_solve_point;i++) {
+      ierr = KSPDestroy(&ctx->ksp[i]);CHKERRQ(ierr);
+    }
+    ierr = PetscFree(ctx->ksp);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
+/* clean PetscSubcomm object when the number of partitions changes */
+PETSC_STATIC_INLINE PetscErrorCode NEPCISSResetSubcomm(NEP nep)
+{
+  PetscErrorCode ierr;
+  NEP_CISS       *ctx = (NEP_CISS*)nep->data;
+
+  PetscFunctionBegin;
+  ierr = NEPCISSResetSolvers(nep);CHKERRQ(ierr);
+  ierr = PetscSubcommDestroy(&ctx->subcomm);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+/* determine whether half of integration points can be avoided (use its conjugates);
+   depends on isreal and the center of the region */
+PETSC_STATIC_INLINE PetscErrorCode NEPCISSSetUseConj(NEP nep,PetscBool *useconj)
+{
+  PetscErrorCode ierr;
+  NEP_CISS       *ctx = (NEP_CISS*)nep->data;
+  PetscScalar    center;
+
+  PetscFunctionBegin;
+  *useconj = PETSC_FALSE;
+  ierr = RGEllipseGetParameters(nep->rg,&center,NULL,NULL);CHKERRQ(ierr);
+  *useconj = (ctx->isreal && PetscImaginaryPart(center) == 0.0)? PETSC_TRUE: PETSC_FALSE;
+  PetscFunctionReturn(0);
+}
+
+/* create PetscSubcomm object and determine num_solve_point (depends on npart, N, useconj) */
+PETSC_STATIC_INLINE PetscErrorCode NEPCISSSetUpSubComm(NEP nep,PetscInt *num_solve_point)
 {
   PetscErrorCode ierr;
   NEP_CISS       *ctx = (NEP_CISS*)nep->data;
   PetscInt       N = ctx->N;
 
   PetscFunctionBegin;
-  if (ctx->useconj) N = N/2;
-  ierr = PetscSubcommDestroy(&ctx->subcomm);CHKERRQ(ierr);
   ierr = PetscSubcommCreate(PetscObjectComm((PetscObject)nep),&ctx->subcomm);CHKERRQ(ierr);
-  ierr = PetscSubcommSetNumber(ctx->subcomm,ctx->num_subcomm);CHKERRQ(ierr);CHKERRQ(ierr);
+  ierr = PetscSubcommSetNumber(ctx->subcomm,ctx->npart);CHKERRQ(ierr);CHKERRQ(ierr);
   ierr = PetscSubcommSetType(ctx->subcomm,PETSC_SUBCOMM_INTERLACED);CHKERRQ(ierr);
   ierr = PetscLogObjectMemory((PetscObject)nep,sizeof(PetscSubcomm));CHKERRQ(ierr);
-  ierr = PetscSubcommSetFromOptions(ctx->subcomm);CHKERRQ(ierr);
   ctx->subcomm_id = ctx->subcomm->color;
-  ctx->num_solve_point = N / ctx->num_subcomm;
-  if ((N%ctx->num_subcomm) > ctx->subcomm_id) ctx->num_solve_point+=1;
+  ierr = NEPCISSSetUseConj(nep,&ctx->useconj);CHKERRQ(ierr);
+  if (ctx->useconj) N = N/2;
+  *num_solve_point = N / ctx->npart;
+  if (N%ctx->npart > ctx->subcomm_id) (*num_solve_point)++;
   PetscFunctionReturn(0);
 }
 
@@ -135,56 +178,29 @@ static PetscErrorCode SolveLinearSystem(NEP nep,Mat T,Mat dT,BV V,PetscInt L_sta
   PetscErrorCode ierr;
   NEP_CISS       *ctx = (NEP_CISS*)nep->data;
   PetscInt       i,j,p_id;
-  Mat            Fz,kspMat;
-  PC             pc;
+  Mat            kspMat;
   Vec            Bvj,vj,yj;
-  KSP            ksp;
 
   PetscFunctionBegin;
-  if (ctx->usest) {
-    ierr = NEPComputeFunction(nep,0,T,T);CHKERRQ(ierr);
-    ierr = MatDuplicate(T,MAT_DO_NOT_COPY_VALUES,&Fz);CHKERRQ(ierr);
-    ierr = KSPCreate(PetscObjectComm((PetscObject)nep),&ksp);CHKERRQ(ierr);
-  }
+  if (!ctx->ksp) { ierr = NEPCISSGetKSPs(nep,&ctx->num_solve_point,&ctx->ksp);CHKERRQ(ierr); }
   ierr = BVCreateVec(V,&Bvj);CHKERRQ(ierr);
   for (i=0;i<ctx->num_solve_point;i++) {
     p_id = i*ctx->subcomm->n + ctx->subcomm_id;
-    ierr = NEPComputeFunction(nep,ctx->omega[p_id],T,T);CHKERRQ(ierr);
-    ierr = NEPComputeJacobian(nep,ctx->omega[p_id],dT);CHKERRQ(ierr);
-    if (!ctx->usest && initksp == PETSC_TRUE) {
+    if (initksp) {
+      ierr = NEPComputeFunction(nep,ctx->omega[p_id],T,T);CHKERRQ(ierr);
       ierr = MatDuplicate(T,MAT_COPY_VALUES,&kspMat);CHKERRQ(ierr);
       ierr = KSPSetOperators(ctx->ksp[i],kspMat,kspMat);CHKERRQ(ierr);
       ierr = MatDestroy(&kspMat);CHKERRQ(ierr);
-      ierr = KSPSetType(ctx->ksp[i],KSPPREONLY);CHKERRQ(ierr);
-      ierr = KSPGetPC(ctx->ksp[i],&pc);CHKERRQ(ierr);
-      ierr = PCSetType(pc,PCLU);CHKERRQ(ierr);
-      ierr = KSPSetFromOptions(ctx->ksp[i]);CHKERRQ(ierr);
-    } else if (ctx->usest) {
-      ierr = MatCopy(T,Fz,DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr);
-      ierr = KSPSetOperators(ksp,Fz,Fz);CHKERRQ(ierr);
-      ierr = KSPSetType(ksp,KSPPREONLY);CHKERRQ(ierr);
-      ierr = KSPGetPC(ksp,&pc);CHKERRQ(ierr);
-      ierr = PCSetType(pc,PCLU);CHKERRQ(ierr);
-      ierr = KSPSetTolerances(ksp,SLEPC_DEFAULT_TOL,PETSC_DEFAULT,PETSC_DEFAULT,PETSC_DEFAULT);CHKERRQ(ierr);
-      ierr = KSPSetFromOptions(ksp);CHKERRQ(ierr);
     }
+    ierr = NEPComputeJacobian(nep,ctx->omega[p_id],dT);CHKERRQ(ierr);
     for (j=L_start;j<L_end;j++) {
       ierr = BVGetColumn(V,j,&vj);CHKERRQ(ierr);
       ierr = BVGetColumn(ctx->Y,i*ctx->L_max+j,&yj);CHKERRQ(ierr);
       ierr = MatMult(dT,vj,Bvj);CHKERRQ(ierr);
-      if (ctx->usest) {
-        ierr = KSPSolve(ksp,Bvj,yj);CHKERRQ(ierr);
-      } else {
-        ierr = KSPSolve(ctx->ksp[i],Bvj,yj);CHKERRQ(ierr);
-      }
+      ierr = KSPSolve(ctx->ksp[i],Bvj,yj);CHKERRQ(ierr);
       ierr = BVRestoreColumn(V,j,&vj);CHKERRQ(ierr);
       ierr = BVRestoreColumn(ctx->Y,i*ctx->L_max+j,&yj);CHKERRQ(ierr);
     }
-    if (ctx->usest && i<ctx->num_solve_point-1) { ierr = KSPReset(ksp);CHKERRQ(ierr); }
-  }
-  if (ctx->usest) {
-    ierr = MatDestroy(&Fz);CHKERRQ(ierr);
-    ierr = KSPDestroy(&ksp);CHKERRQ(ierr);
   }
   ierr = VecDestroy(&Bvj);CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -415,20 +431,25 @@ PetscErrorCode NEPSetUp_CISS(NEP nep)
 {
   PetscErrorCode ierr;
   NEP_CISS       *ctx = (NEP_CISS*)nep->data;
-  PetscInt       i,nwork;
-  PetscBool      istrivial,isellipse,flg;
-  PetscScalar    center;
+  PetscInt       nwork;
+  PetscBool      istrivial,isellipse,flg,useconj;
 
   PetscFunctionBegin;
-  if (!nep->ncv) nep->ncv = ctx->L_max*ctx->M;
-  else {
+  if (!nep->ncv) {
+    nep->ncv = ctx->L_max*ctx->M;
+    if (nep->ncv>nep->n) {
+      nep->ncv = nep->n;
+      ctx->L_max = nep->ncv/ctx->M;
+      if (!ctx->L_max) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_SUP,"Cannot adjust solver parameters, try setting a smaller value of M (moment size)");
+    }
+  } else {
     ctx->L_max = nep->ncv/ctx->M;
-    if (ctx->L_max == 0) {
+    if (!ctx->L_max) {
       ctx->L_max = 1;
       nep->ncv = ctx->L_max*ctx->M;
     }
-    if (ctx->L > ctx->L_max) ctx->L = ctx->L_max;
   }
+  ctx->L = PetscMin(ctx->L,ctx->L_max);
   if (!nep->max_it) nep->max_it = 1;
   if (!nep->mpd) nep->mpd = nep->ncv;
   if (!nep->which) nep->which = NEP_ALL;
@@ -440,14 +461,13 @@ PetscErrorCode NEPSetUp_CISS(NEP nep)
   ierr = RGGetComplement(nep->rg,&flg);CHKERRQ(ierr);
   if (flg) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_SUP,"A region with complement flag set is not allowed");
   ierr = PetscObjectTypeCompare((PetscObject)nep->rg,RGELLIPSE,&isellipse);CHKERRQ(ierr);
-  if (!isellipse) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_SUP,"Currently only implemented for elliptic or arc regions");
-  ierr = RGEllipseGetParameters(nep->rg,&center,NULL,NULL);CHKERRQ(ierr);
-  if (ctx->isreal && PetscImaginaryPart(center) == 0.0) ctx->useconj = PETSC_TRUE;
-  else ctx->useconj = PETSC_FALSE;
+  if (!isellipse) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_SUP,"Currently only implemented for elliptic regions");
+  /* if useconj has changed, then reset subcomm data */
+  ierr = NEPCISSSetUseConj(nep,&useconj);CHKERRQ(ierr);
+  if (useconj!=ctx->useconj) { ierr = NEPCISSResetSubcomm(nep);CHKERRQ(ierr); }
 
   /* create split comm */
-  ctx->num_subcomm = 1;
-  ierr = SetSolverComm(nep);CHKERRQ(ierr);
+  if (!ctx->subcomm) { ierr = NEPCISSSetUpSubComm(nep,&ctx->num_solve_point);CHKERRQ(ierr); }
 
   ierr = NEPAllocateSolution(nep,0);CHKERRQ(ierr);
   if (ctx->weight) { ierr = PetscFree4(ctx->weight,ctx->omega,ctx->pp,ctx->sigma);CHKERRQ(ierr); }
@@ -462,26 +482,6 @@ PetscErrorCode NEPSetUp_CISS(NEP nep)
   ierr = BVDuplicateResize(nep->V,ctx->L_max,&ctx->V);CHKERRQ(ierr);
   ierr = PetscLogObjectParent((PetscObject)nep,(PetscObject)ctx->V);CHKERRQ(ierr);
 
-  if (!ctx->usest) {
-    if (ctx->ksp) {
-      for (i=0;i<ctx->num_solve_point;i++) {
-        ierr = KSPDestroy(&ctx->ksp[i]);CHKERRQ(ierr);
-      }
-      ierr = PetscFree(ctx->ksp);CHKERRQ(ierr);
-    }
-    ierr = PetscMalloc1(ctx->num_solve_point,&ctx->ksp);CHKERRQ(ierr);
-    ierr = PetscLogObjectMemory((PetscObject)nep,ctx->num_solve_point*sizeof(KSP));CHKERRQ(ierr);
-    for (i=0;i<ctx->num_solve_point;i++) {
-      ierr = KSPCreate(PetscSubcommChild(ctx->subcomm),&ctx->ksp[i]);CHKERRQ(ierr);
-      ierr = PetscObjectIncrementTabLevel((PetscObject)ctx->ksp[i],(PetscObject)nep,1);CHKERRQ(ierr);
-      ierr = PetscLogObjectParent((PetscObject)nep,(PetscObject)ctx->ksp[i]);CHKERRQ(ierr);
-      ierr = KSPSetOptionsPrefix(ctx->ksp[i],((PetscObject)nep)->prefix);CHKERRQ(ierr);
-      ierr = KSPAppendOptionsPrefix(ctx->ksp[i],"nep_ciss_");CHKERRQ(ierr);
-      ierr = KSPSetErrorIfNotConverged(ctx->ksp[i],PETSC_TRUE);CHKERRQ(ierr);
-      ierr = KSPSetTolerances(ctx->ksp[i],SLEPC_DEFAULT_TOL,PETSC_DEFAULT,PETSC_DEFAULT,PETSC_DEFAULT);CHKERRQ(ierr);
-    }
-  }
-
   ierr = BVDestroy(&ctx->Y);CHKERRQ(ierr);
   ierr = BVDuplicateResize(nep->V,ctx->num_solve_point*ctx->L_max,&ctx->Y);CHKERRQ(ierr);
 
@@ -575,7 +575,6 @@ PetscErrorCode NEPSolve_CISS(NEP nep)
     ierr = DSRestoreArray(nep->ds,DS_MAT_B,&temp);CHKERRQ(ierr);
     ierr = DSSolve(nep->ds,nep->eigr,nep->eigi);CHKERRQ(ierr);
     ierr = DSSynchronize(nep->ds,nep->eigr,nep->eigi);CHKERRQ(ierr);
-    ierr = DSVectors(nep->ds,DS_MAT_X,NULL,NULL);CHKERRQ(ierr);
     for (i=0;i<nv;i++){
       nep->eigr[i] = (nep->eigr[i]*radius+center)*rgscale;
 #if !defined(PETSC_USE_COMPLEX)
@@ -645,9 +644,12 @@ PetscErrorCode NEPSolve_CISS(NEP nep)
 
 static PetscErrorCode NEPCISSSetSizes_CISS(NEP nep,PetscInt ip,PetscInt bs,PetscInt ms,PetscInt npart,PetscInt bsmax,PetscBool realmats)
 {
-  NEP_CISS *ctx = (NEP_CISS*)nep->data;
+  PetscErrorCode ierr;
+  NEP_CISS       *ctx = (NEP_CISS*)nep->data;
+  PetscInt       oN,onpart;
 
   PetscFunctionBegin;
+  oN = ctx->N;
   if (ip == PETSC_DECIDE || ip == PETSC_DEFAULT) {
     if (ctx->N!=32) { ctx->N =32; ctx->M = ctx->N/4; }
   } else {
@@ -659,7 +661,6 @@ static PetscErrorCode NEPCISSSetSizes_CISS(NEP nep,PetscInt ip,PetscInt bs,Petsc
     ctx->L = 16;
   } else {
     if (bs<1) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"The bs argument must be > 0");
-    if (bs>ctx->L_max) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"The bs argument must be less than or equal to the maximum number of block size");
     ctx->L = bs;
   }
   if (ms == PETSC_DECIDE || ms == PETSC_DEFAULT) {
@@ -669,19 +670,21 @@ static PetscErrorCode NEPCISSSetSizes_CISS(NEP nep,PetscInt ip,PetscInt bs,Petsc
     if (ms>ctx->N) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"The ms argument must be less than or equal to the number of integration points");
     ctx->M = ms;
   }
+  onpart = ctx->npart;
   if (npart == PETSC_DECIDE || npart == PETSC_DEFAULT) {
-    ctx->num_subcomm = 1;
+    ctx->npart = 1;
   } else {
     if (npart<1) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"The npart argument must be > 0");
-    ctx->num_subcomm = npart;
+    ctx->npart = npart;
+    if (npart>1) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_SUP,"Current implementation does not allow partitions");
   }
   if (bsmax == PETSC_DECIDE || bsmax == PETSC_DEFAULT) {
-    ctx->L = 256;
+    ctx->L_max = 64;
   } else {
     if (bsmax<1) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"The bsmax argument must be > 0");
-    if (bsmax<ctx->L) ctx->L_max = ctx->L;
-    else ctx->L_max = bsmax;
+    ctx->L_max = PetscMax(bsmax,ctx->L);
   }
+  if (onpart != ctx->npart || oN != ctx->N || realmats != ctx->isreal) { ierr = NEPCISSResetSubcomm(nep);CHKERRQ(ierr); }
   ctx->isreal = realmats;
   nep->state = NEP_STATE_INITIAL;
   PetscFunctionReturn(0);
@@ -746,7 +749,7 @@ static PetscErrorCode NEPCISSGetSizes_CISS(NEP nep,PetscInt *ip,PetscInt *bs,Pet
   if (ip) *ip = ctx->N;
   if (bs) *bs = ctx->L;
   if (ms) *ms = ctx->M;
-  if (npart) *npart = ctx->num_subcomm;
+  if (npart) *npart = ctx->npart;
   if (bsmax) *bsmax = ctx->L_max;
   if (realmats) *realmats = ctx->isreal;
   PetscFunctionReturn(0);
@@ -958,6 +961,70 @@ PetscErrorCode NEPCISSGetRefinement(NEP nep, PetscInt *inner, PetscInt *blsize)
   PetscFunctionReturn(0);
 }
 
+static PetscErrorCode NEPCISSGetKSPs_CISS(NEP nep,PetscInt *nsolve,KSP **ksp)
+{
+  PetscErrorCode ierr;
+  NEP_CISS       *ctx = (NEP_CISS*)nep->data;
+  PetscInt       i;
+  PC             pc;
+
+  PetscFunctionBegin;
+  if (!ctx->ksp) {
+    if (!ctx->subcomm) {  /* initialize subcomm first */
+      ierr = NEPCISSSetUseConj(nep,&ctx->useconj);CHKERRQ(ierr);
+      ierr = NEPCISSSetUpSubComm(nep,&ctx->num_solve_point);CHKERRQ(ierr);
+    }
+    ierr = PetscMalloc1(ctx->num_solve_point,&ctx->ksp);CHKERRQ(ierr);
+    for (i=0;i<ctx->num_solve_point;i++) {
+      ierr = KSPCreate(PetscSubcommChild(ctx->subcomm),&ctx->ksp[i]);CHKERRQ(ierr);
+      ierr = PetscObjectIncrementTabLevel((PetscObject)ctx->ksp[i],(PetscObject)nep,1);CHKERRQ(ierr);
+      ierr = KSPSetOptionsPrefix(ctx->ksp[i],((PetscObject)nep)->prefix);CHKERRQ(ierr);
+      ierr = KSPAppendOptionsPrefix(ctx->ksp[i],"nep_ciss_");CHKERRQ(ierr);
+      ierr = PetscLogObjectParent((PetscObject)nep,(PetscObject)ctx->ksp[i]);CHKERRQ(ierr);
+      ierr = KSPSetErrorIfNotConverged(ctx->ksp[i],PETSC_TRUE);CHKERRQ(ierr);
+      ierr = KSPSetTolerances(ctx->ksp[i],SLEPC_DEFAULT_TOL,PETSC_DEFAULT,PETSC_DEFAULT,PETSC_DEFAULT);CHKERRQ(ierr);
+      ierr = KSPGetPC(ctx->ksp[i],&pc);CHKERRQ(ierr);
+      ierr = KSPSetType(ctx->ksp[i],KSPPREONLY);CHKERRQ(ierr);
+      ierr = PCSetType(pc,PCLU);CHKERRQ(ierr);
+    }
+  }
+  if (nsolve) *nsolve = ctx->num_solve_point;
+  if (ksp)    *ksp    = ctx->ksp;
+  PetscFunctionReturn(0);
+}
+
+/*@C
+   NEPCISSGetKSPs - Retrieve the array of linear solver objects associated with
+   the CISS solver.
+
+   Not Collective
+
+   Input Parameter:
+.  nep - nonlinear eigenvalue solver
+
+   Output Parameters:
++  nsolve - number of solver objects
+-  ksp - array of linear solver object
+
+   Notes:
+   The number of KSP solvers is equal to the number of integration points divided by
+   the number of partitions. This value is halved in the case of real matrices with
+   a region centered at the real axis.
+
+   Level: advanced
+
+.seealso: NEPCISSSetSizes()
+@*/
+PetscErrorCode NEPCISSGetKSPs(NEP nep,PetscInt *nsolve,KSP **ksp)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
+  ierr = PetscUseMethod(nep,"NEPCISSGetKSPs_C",(NEP,PetscInt*,KSP**),(nep,nsolve,ksp));CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 PetscErrorCode NEPReset_CISS(NEP nep)
 {
   PetscErrorCode ierr;
@@ -968,11 +1035,8 @@ PetscErrorCode NEPReset_CISS(NEP nep)
   ierr = BVDestroy(&ctx->S);CHKERRQ(ierr);
   ierr = BVDestroy(&ctx->V);CHKERRQ(ierr);
   ierr = BVDestroy(&ctx->Y);CHKERRQ(ierr);
-  if (!ctx->usest) {
-    for (i=0;i<ctx->num_solve_point;i++) {
-      ierr = KSPDestroy(&ctx->ksp[i]);CHKERRQ(ierr);
-    }
-    ierr = PetscFree(ctx->ksp);CHKERRQ(ierr);
+  for (i=0;i<ctx->num_solve_point;i++) {
+    ierr = KSPReset(ctx->ksp[i]);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -980,8 +1044,9 @@ PetscErrorCode NEPReset_CISS(NEP nep)
 PetscErrorCode NEPSetFromOptions_CISS(PetscOptionItems *PetscOptionsObject,NEP nep)
 {
   PetscErrorCode ierr;
+  NEP_CISS       *ctx = (NEP_CISS*)nep->data;
   PetscReal      r1,r2;
-  PetscInt       i1,i2,i3,i4,i5,i6,i7;
+  PetscInt       i,i1,i2,i3,i4,i5,i6,i7;
   PetscBool      b1;
 
   PetscFunctionBegin;
@@ -1007,6 +1072,14 @@ PetscErrorCode NEPSetFromOptions_CISS(PetscOptionItems *PetscOptionsObject,NEP n
     ierr = NEPCISSSetRefinement(nep,i6,i7);CHKERRQ(ierr);
 
   ierr = PetscOptionsTail();CHKERRQ(ierr);
+
+  if (!nep->rg) { ierr = NEPGetRG(nep,&nep->rg);CHKERRQ(ierr); }
+  ierr = RGSetFromOptions(nep->rg);CHKERRQ(ierr); /* this is necessary here to set useconj */
+  if (!ctx->ksp) { ierr = NEPCISSGetKSPs(nep,&ctx->num_solve_point,&ctx->ksp);CHKERRQ(ierr); }
+  for (i=0;i<ctx->num_solve_point;i++) {
+    ierr = KSPSetFromOptions(ctx->ksp[i]);CHKERRQ(ierr);
+  }
+  ierr = PetscSubcommSetFromOptions(ctx->subcomm);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -1016,7 +1089,7 @@ PetscErrorCode NEPDestroy_CISS(NEP nep)
   NEP_CISS       *ctx = (NEP_CISS*)nep->data;
 
   PetscFunctionBegin;
-  ierr = PetscSubcommDestroy(&ctx->subcomm);CHKERRQ(ierr);
+  ierr = NEPCISSResetSubcomm(nep);CHKERRQ(ierr);
   ierr = PetscFree4(ctx->weight,ctx->omega,ctx->pp,ctx->sigma);CHKERRQ(ierr);
   ierr = PetscFree(nep->data);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPCISSSetSizes_C",NULL);CHKERRQ(ierr);
@@ -1025,6 +1098,7 @@ PetscErrorCode NEPDestroy_CISS(NEP nep)
   ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPCISSGetThreshold_C",NULL);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPCISSSetRefinement_C",NULL);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPCISSGetRefinement_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPCISSGetKSPs_C",NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -1037,15 +1111,14 @@ PetscErrorCode NEPView_CISS(NEP nep,PetscViewer viewer)
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
-    ierr = PetscViewerASCIIPrintf(viewer,"  sizes { integration points: %D, block size: %D, moment size: %D, partitions: %D, maximum block size: %D }\n",ctx->N,ctx->L,ctx->M,ctx->num_subcomm,ctx->L_max);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  sizes { integration points: %D, block size: %D, moment size: %D, partitions: %D, maximum block size: %D }\n",ctx->N,ctx->L,ctx->M,ctx->npart,ctx->L_max);CHKERRQ(ierr);
     if (ctx->isreal) {
       ierr = PetscViewerASCIIPrintf(viewer,"  exploiting symmetry of integration points\n");CHKERRQ(ierr);
     }
     ierr = PetscViewerASCIIPrintf(viewer,"  threshold { delta: %g, spurious threshold: %g }\n",(double)ctx->delta,(double)ctx->spurious_threshold);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  iterative refinement  { inner: %D, blocksize: %D }\n",ctx->refine_inner, ctx->refine_blocksize);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
-    if (!ctx->usest && ctx->ksp[0]) { ierr = KSPView(ctx->ksp[0],viewer);CHKERRQ(ierr); }
-    ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
+    if (!ctx->ksp) { ierr = NEPCISSGetKSPs(nep,&ctx->num_solve_point,&ctx->ksp);CHKERRQ(ierr); }
+    ierr = KSPView(ctx->ksp[0],viewer);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -1065,9 +1138,8 @@ PETSC_EXTERN PetscErrorCode NEPCreate_CISS(NEP nep)
   ctx->delta              = 1e-12;
   ctx->L_max              = 64;
   ctx->spurious_threshold = 1e-4;
-  ctx->usest              = PETSC_FALSE;
   ctx->isreal             = PETSC_FALSE;
-  ctx->num_subcomm        = 1;
+  ctx->npart              = 1;
 
   nep->ops->solve          = NEPSolve_CISS;
   nep->ops->setup          = NEPSetUp_CISS;
@@ -1082,6 +1154,7 @@ PETSC_EXTERN PetscErrorCode NEPCreate_CISS(NEP nep)
   ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPCISSGetThreshold_C",NEPCISSGetThreshold_CISS);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPCISSSetRefinement_C",NEPCISSSetRefinement_CISS);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPCISSGetRefinement_C",NEPCISSGetRefinement_CISS);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)nep,"NEPCISSGetKSPs_C",NEPCISSGetKSPs_CISS);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
