@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -276,6 +276,7 @@ PetscErrorCode PEPSetUp_Linear(PEP pep)
     if (sinv) pep->which = PEP_TARGET_MAGNITUDE;
     else pep->which = PEP_LARGEST_MAGNITUDE;
   }
+  if (pep->which==PEP_ALL) SETERRQ(PetscObjectComm((PetscObject)pep),1,"Wrong value of pep->which");
   ierr = STSetUp(pep->st);CHKERRQ(ierr);
   if (!ctx->eps) { ierr = PEPLinearGetEPS(pep,&ctx->eps);CHKERRQ(ierr); }
   ierr = EPSGetST(ctx->eps,&st);CHKERRQ(ierr);
@@ -306,7 +307,8 @@ PetscErrorCode PEPSetUp_Linear(PEP pep)
 
     switch (pep->problem_type) {
       case PEP_GENERAL:    i = 0; break;
-      case PEP_HERMITIAN:  i = 2; break;
+      case PEP_HERMITIAN:
+      case PEP_HYPERBOLIC: i = 2; break;
       case PEP_GYROSCOPIC: i = 4; break;
     }
     i += ctx->cform-1;
@@ -386,6 +388,7 @@ PetscErrorCode PEPSetUp_Linear(PEP pep)
           case PEP_TARGET_MAGNITUDE:   which = EPS_TARGET_MAGNITUDE; break;
           case PEP_TARGET_REAL:        which = EPS_TARGET_REAL; break;
           case PEP_TARGET_IMAGINARY:   which = EPS_TARGET_IMAGINARY; break;
+          case PEP_ALL:                which = EPS_ALL; break;
           case PEP_WHICH_USER:         which = EPS_WHICH_USER;
             ierr = EPSSetEigenvalueComparison(ctx->eps,pep->sc->comparison,pep->sc->comparisonctx);CHKERRQ(ierr);
             break;
@@ -957,9 +960,9 @@ static PetscErrorCode PEPLinearGetEPS_Linear(PEP pep,EPS *eps)
   PetscFunctionBegin;
   if (!ctx->eps) {
     ierr = EPSCreate(PetscObjectComm((PetscObject)pep),&ctx->eps);CHKERRQ(ierr);
+    ierr = PetscObjectIncrementTabLevel((PetscObject)ctx->eps,(PetscObject)pep,1);CHKERRQ(ierr);
     ierr = EPSSetOptionsPrefix(ctx->eps,((PetscObject)pep)->prefix);CHKERRQ(ierr);
     ierr = EPSAppendOptionsPrefix(ctx->eps,"pep_linear_");CHKERRQ(ierr);
-    ierr = PetscObjectIncrementTabLevel((PetscObject)ctx->eps,(PetscObject)pep,1);CHKERRQ(ierr);
     ierr = PetscLogObjectParent((PetscObject)pep,(PetscObject)ctx->eps);CHKERRQ(ierr);
     ierr = EPSMonitorSet(ctx->eps,EPSMonitor_Linear,pep,NULL);CHKERRQ(ierr);
   }
@@ -1006,9 +1009,7 @@ PetscErrorCode PEPView_Linear(PEP pep,PetscViewer viewer)
     if (!ctx->eps) { ierr = PEPLinearGetEPS(pep,&ctx->eps);CHKERRQ(ierr); }
     ierr = PetscViewerASCIIPrintf(viewer,"  %s matrices\n",ctx->explicitmatrix? "explicit": "implicit");CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  %s companion form\n",ctx->cform==1? "1st": "2nd");CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
     ierr = EPSView(ctx->eps,viewer);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
