@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -109,7 +109,7 @@ PetscErrorCode PEPSetFromOptions(PEP pep)
   PetscErrorCode  ierr;
   char            type[256];
   PetscBool       set,flg,flg1,flg2,flg3,flg4,flg5;
-  PetscReal       r,t;
+  PetscReal       r,t,array[2]={0,0};
   PetscScalar     s;
   PetscInt        i,j,k;
   PetscDrawLG     lg;
@@ -132,6 +132,8 @@ PetscErrorCode PEPSetFromOptions(PEP pep)
     if (flg) { ierr = PEPSetProblemType(pep,PEP_GENERAL);CHKERRQ(ierr); }
     ierr = PetscOptionsBoolGroup("-pep_hermitian","Hermitian polynomial eigenvalue problem","PEPSetProblemType",&flg);CHKERRQ(ierr);
     if (flg) { ierr = PEPSetProblemType(pep,PEP_HERMITIAN);CHKERRQ(ierr); }
+    ierr = PetscOptionsBoolGroup("-pep_hyperbolic","Hyperbolic polynomial eigenvalue problem","PEPSetProblemType",&flg);CHKERRQ(ierr);
+    if (flg) { ierr = PEPSetProblemType(pep,PEP_HYPERBOLIC);CHKERRQ(ierr); }
     ierr = PetscOptionsBoolGroupEnd("-pep_gyroscopic","Gyroscopic polynomial eigenvalue problem","PEPSetProblemType",&flg);CHKERRQ(ierr);
     if (flg) { ierr = PEPSetProblemType(pep,PEP_GYROSCOPIC);CHKERRQ(ierr); }
 
@@ -217,6 +219,14 @@ PetscErrorCode PEPSetFromOptions(PEP pep)
       ierr = PEPSetTarget(pep,s);CHKERRQ(ierr);
     }
 
+    k = 2;
+    ierr = PetscOptionsRealArray("-pep_interval","Computational interval (two real values separated with a comma without spaces)","PEPSetInterval",array,&k,&flg);CHKERRQ(ierr);
+    if (flg) {
+      if (k<2) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_SIZ,"Must pass two values in -pep_interval (comma-separated without spaces)");
+      ierr = PEPSetWhichEigenpairs(pep,PEP_ALL);CHKERRQ(ierr);
+      ierr = PEPSetInterval(pep,array[0],array[1]);CHKERRQ(ierr);
+    }
+
     /* -----------------------------------------------------------------------*/
     /*
       Cancels all monitors hardwired into code before call to PEPSetFromOptions()
@@ -410,9 +420,12 @@ PetscErrorCode PEPGetDimensions(PEP pep,PetscInt *nev,PetscInt *ncv,PetscInt *mp
    ncv=nev+mpd. If nev is not too large, mpd=nev is a reasonable choice, otherwise
    a smaller value should be used.
 
+   When computing all eigenvalues in an interval, see PEPSetInterval(), these
+   parameters lose relevance, and tuning must be done with PEPSTOARSetDimensions().
+
    Level: intermediate
 
-.seealso: PEPGetDimensions()
+.seealso: PEPGetDimensions(), PEPSetInterval(), PEPSTOARSetDimensions()
 @*/
 PetscErrorCode PEPSetDimensions(PEP pep,PetscInt nev,PetscInt ncv,PetscInt mpd)
 {
@@ -461,6 +474,7 @@ PetscErrorCode PEPSetDimensions(PEP pep,PetscInt nev,PetscInt ncv,PetscInt mpd)
 .     PEP_TARGET_MAGNITUDE - eigenvalues closest to the target (in magnitude)
 .     PEP_TARGET_REAL - eigenvalues with real part closest to target
 .     PEP_TARGET_IMAGINARY - eigenvalues with imaginary part closest to target
+.     PEP_ALL - all eigenvalues contained in a given interval
 -     PEP_WHICH_USER - user defined ordering set with PEPSetEigenvalueComparison()
 
    Options Database Keys:
@@ -472,7 +486,8 @@ PetscErrorCode PEPSetDimensions(PEP pep,PetscInt nev,PetscInt ncv,PetscInt mpd)
 .   -pep_smallest_imaginary - Sets smallest imaginary parts
 .   -pep_target_magnitude - Sets eigenvalues closest to target
 .   -pep_target_real - Sets real parts closest to target
--   -pep_target_imaginary - Sets imaginary parts closest to target
+.   -pep_target_imaginary - Sets imaginary parts closest to target
+-   -pep_all - Sets all eigenvalues in an interval
 
    Notes:
    Not all eigensolvers implemented in PEP account for all the possible values
@@ -485,9 +500,15 @@ PetscErrorCode PEPSetDimensions(PEP pep,PetscInt nev,PetscInt ncv,PetscInt mpd)
    The criterion PEP_TARGET_IMAGINARY is available only in case PETSc and
    SLEPc have been built with complex scalars.
 
+   PEP_ALL is intended for use in combination with an interval (see
+   PEPSetInterval()), when all eigenvalues within the interval are requested.
+   In that case, the number of eigenvalues is unknown, so the nev parameter
+   has a different sense, see PEPSetDimensions().
+
    Level: intermediate
 
-.seealso: PEPGetWhichEigenpairs(), PEPSetTarget(), PEPSetEigenvalueComparison(), PEPWhich
+.seealso: PEPGetWhichEigenpairs(), PEPSetTarget(), PEPSetInterval(),
+          PEPSetDimensions(), PEPSetEigenvalueComparison(), PEPWhich
 @*/
 PetscErrorCode PEPSetWhichEigenpairs(PEP pep,PEPWhich which)
 {
@@ -506,6 +527,7 @@ PetscErrorCode PEPSetWhichEigenpairs(PEP pep,PEPWhich which)
 #if defined(PETSC_USE_COMPLEX)
     case PEP_TARGET_IMAGINARY:
 #endif
+    case PEP_ALL:
     case PEP_WHICH_USER:
       if (pep->which != which) {
         pep->state = PEP_STATE_INITIAL;
@@ -599,18 +621,21 @@ PetscErrorCode PEPSetEigenvalueComparison(PEP pep,PetscErrorCode (*func)(PetscSc
    Options Database Keys:
 +  -pep_general - general problem with no particular structure
 .  -pep_hermitian - problem whose coefficient matrices are Hermitian
+.  -pep_hyperbolic - Hermitian problem that satisfies the definition of hyperbolic
 -  -pep_gyroscopic - problem with Hamiltonian structure
 
    Notes:
    Allowed values for the problem type are: general (PEP_GENERAL), Hermitian
-   (PEP_HERMITIAN), and gyroscopic (PEP_GYROSCOPIC).
+   (PEP_HERMITIAN), hyperbolic (PEP_HYPERBOLIC), and gyroscopic (PEP_GYROSCOPIC).
 
    This function is used to instruct SLEPc to exploit certain structure in
    the polynomial eigenproblem. By default, no particular structure is assumed.
 
    If the problem matrices are Hermitian (symmetric in the real case) or
    Hermitian/skew-Hermitian then the solver can exploit this fact to perform
-   less operations or provide better stability.
+   less operations or provide better stability. Hyperbolic problems are a
+   particular case of Hermitian problems, some solvers may treat them simply as
+   Hermitian.
 
    Level: intermediate
 
@@ -621,7 +646,7 @@ PetscErrorCode PEPSetProblemType(PEP pep,PEPProblemType type)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
   PetscValidLogicalCollectiveEnum(pep,type,2);
-  if (type!=PEP_GENERAL && type!=PEP_HERMITIAN && type!=PEP_GYROSCOPIC) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_WRONG,"Unknown eigenvalue problem type");
+  if (type!=PEP_GENERAL && type!=PEP_HERMITIAN && type!=PEP_HYPERBOLIC && type!=PEP_GYROSCOPIC) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_WRONG,"Unknown eigenvalue problem type");
   if (type != pep->problem_type) {
     pep->problem_type = type;
     pep->state = PEP_STATE_INITIAL;
