@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -71,7 +71,7 @@ PetscErrorCode PEPSetUp(PEP pep)
   KSP            ksp;
   PC             pc;
   PetscMPIInt    size;
-  const MatSolverPackage stype;
+  MatSolverType  stype;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
@@ -127,7 +127,7 @@ PetscErrorCode PEPSetUp(PEP pep)
       if (!flg) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"The MBE scheme for refinement requires a direct solver in KSP");
       ierr = MPI_Comm_size(PetscObjectComm((PetscObject)pc),&size);CHKERRQ(ierr);
       if (size>1) {   /* currently selected PC is a factorization */
-        ierr = PCFactorGetMatSolverPackage(pc,&stype);CHKERRQ(ierr);
+        ierr = PCFactorGetMatSolverType(pc,&stype);CHKERRQ(ierr);
         ierr = PetscStrcmp(stype,MATSOLVERPETSC,&flg);CHKERRQ(ierr);
         if (flg) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"For Newton refinement, you chose to solve linear systems with a factorization, but in parallel runs you need to select an external package");
       }
@@ -191,6 +191,10 @@ PetscErrorCode PEPSetUp(PEP pep)
       pep->sc->comparisonctx = &pep->target;
 #endif
       break;
+    case PEP_ALL:
+      pep->sc->comparison    = SlepcCompareSmallestReal;
+      pep->sc->comparisonctx = NULL;
+      break;
     case PEP_WHICH_USER:
       break;
   }
@@ -198,14 +202,15 @@ PetscErrorCode PEPSetUp(PEP pep)
   pep->sc->mapobj = NULL;
 
   /* fill sorting criterion for DS */
-  ierr = DSGetSlepcSC(pep->ds,&sc);CHKERRQ(ierr);
-  ierr = RGIsTrivial(pep->rg,&istrivial);CHKERRQ(ierr);
-  sc->rg            = istrivial? NULL: pep->rg;
-  sc->comparison    = pep->sc->comparison;
-  sc->comparisonctx = pep->sc->comparisonctx;
-  sc->map           = SlepcMap_ST;
-  sc->mapobj        = (PetscObject)pep->st;
-
+  if (pep->which!=PEP_ALL) {
+    ierr = DSGetSlepcSC(pep->ds,&sc);CHKERRQ(ierr);
+    ierr = RGIsTrivial(pep->rg,&istrivial);CHKERRQ(ierr);
+    sc->rg            = istrivial? NULL: pep->rg;
+    sc->comparison    = pep->sc->comparison;
+    sc->comparisonctx = pep->sc->comparisonctx;
+    sc->map           = SlepcMap_ST;
+    sc->mapobj        = (PetscObject)pep->st;
+  }
   /* setup ST */
   ierr = STSetUp(pep->st);CHKERRQ(ierr);
 
