@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -183,33 +183,18 @@ PetscErrorCode BVMatMult_Contiguous(BV V,Mat A,BV W)
 {
   PetscErrorCode ierr;
   BV_CONTIGUOUS  *v = (BV_CONTIGUOUS*)V->data,*w = (BV_CONTIGUOUS*)W->data;
-  PetscScalar    *pb,*pc;
-  PetscInt       j,m;
+  PetscInt       j;
   PetscBool      flg;
+  Mat            Vmat,Wmat;
 
   PetscFunctionBegin;
   ierr = MatHasOperation(A,MATOP_MAT_MULT,&flg);CHKERRQ(ierr);
   if (V->vmm && flg) {
-    m = V->k-V->l;
-    if (V->vmm==BV_MATMULT_MAT_SAVE) {
-      ierr = BV_AllocateMatMult(V,A,m);CHKERRQ(ierr);
-      ierr = MatDenseGetArray(V->B,&pb);CHKERRQ(ierr);
-      ierr = PetscMemcpy(pb,v->array+(V->nc+V->l)*V->n,m*V->n*sizeof(PetscScalar));CHKERRQ(ierr);
-      ierr = MatDenseRestoreArray(V->B,&pb);CHKERRQ(ierr);
-    } else {  /* BV_MATMULT_MAT */
-      ierr = MatCreateDense(PetscObjectComm((PetscObject)V),V->n,PETSC_DECIDE,V->N,m,v->array+(V->nc+V->l)*V->n,&V->B);CHKERRQ(ierr);
-    }
-    if (!V->C) {
-      ierr = MatMatMultSymbolic(A,V->B,PETSC_DEFAULT,&V->C);CHKERRQ(ierr);
-    }
-    ierr = MatMatMultNumeric(A,V->B,V->C);CHKERRQ(ierr);
-    ierr = MatDenseGetArray(V->C,&pc);CHKERRQ(ierr);
-    ierr = PetscMemcpy(w->array+(W->nc+W->l)*W->n,pc,m*V->n*sizeof(PetscScalar));CHKERRQ(ierr);
-    ierr = MatDenseRestoreArray(V->C,&pc);CHKERRQ(ierr);
-    if (V->vmm==BV_MATMULT_MAT) {
-      ierr = MatDestroy(&V->B);CHKERRQ(ierr);
-      ierr = MatDestroy(&V->C);CHKERRQ(ierr);
-    }
+    ierr = BVGetMat(V,&Vmat);CHKERRQ(ierr);
+    ierr = BVGetMat(W,&Wmat);CHKERRQ(ierr);
+    ierr = MatMatMult(A,Vmat,MAT_REUSE_MATRIX,PETSC_DEFAULT,&Wmat);CHKERRQ(ierr);
+    ierr = BVRestoreMat(V,&Vmat);CHKERRQ(ierr);
+    ierr = BVRestoreMat(W,&Wmat);CHKERRQ(ierr);
   } else {
     for (j=0;j<V->k-V->l;j++) {
       ierr = MatMult(A,v->V[V->nc+V->l+j],w->V[W->nc+W->l+j]);CHKERRQ(ierr);
@@ -231,41 +216,59 @@ PetscErrorCode BVCopy_Contiguous(BV V,BV W)
   PetscFunctionReturn(0);
 }
 
+PetscErrorCode BVCopyColumn_Contiguous(BV V,PetscInt j,PetscInt i)
+{
+  PetscErrorCode ierr;
+  BV_CONTIGUOUS  *v = (BV_CONTIGUOUS*)V->data;
+
+  PetscFunctionBegin;
+  ierr = PetscMemcpy(v->array+(V->nc+i)*V->n,v->array+(V->nc+j)*V->n,V->n*sizeof(PetscScalar));CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 PetscErrorCode BVResize_Contiguous(BV bv,PetscInt m,PetscBool copy)
 {
   PetscErrorCode ierr;
   BV_CONTIGUOUS  *ctx = (BV_CONTIGUOUS*)bv->data;
-  PetscInt       j,bs;
+  PetscInt       j,bs,lsplit;
   PetscScalar    *newarray;
   Vec            *newV;
   char           str[50];
+  BV             parent;
 
   PetscFunctionBegin;
-  ierr = VecGetBlockSize(bv->t,&bs);CHKERRQ(ierr);
-  ierr = PetscMalloc1(m*bv->n,&newarray);CHKERRQ(ierr);
-  ierr = PetscMemzero(newarray,m*bv->n*sizeof(PetscScalar));CHKERRQ(ierr);
-  ierr = PetscMalloc1(m,&newV);CHKERRQ(ierr);
-  for (j=0;j<m;j++) {
-    if (ctx->mpi) {
-      ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)bv->t),bs,bv->n,PETSC_DECIDE,newarray+j*bv->n,newV+j);CHKERRQ(ierr);
-    } else {
-      ierr = VecCreateSeqWithArray(PetscObjectComm((PetscObject)bv->t),bs,bv->n,newarray+j*bv->n,newV+j);CHKERRQ(ierr);
-    }
-  }
-  ierr = PetscLogObjectParents(bv,m,newV);CHKERRQ(ierr);
-  if (((PetscObject)bv)->name) {
+  if (bv->issplit==2) {
+    parent = bv->splitparent;
+    lsplit = parent->lsplit;
+    ctx->V = ((BV_CONTIGUOUS*)parent->data)->V+lsplit;
+    ctx->array = ((BV_CONTIGUOUS*)parent->data)->array+lsplit*bv->n;
+  } else if (!bv->issplit) {
+    ierr = VecGetBlockSize(bv->t,&bs);CHKERRQ(ierr);
+    ierr = PetscMalloc1(m*bv->n,&newarray);CHKERRQ(ierr);
+    ierr = PetscMemzero(newarray,m*bv->n*sizeof(PetscScalar));CHKERRQ(ierr);
+    ierr = PetscMalloc1(m,&newV);CHKERRQ(ierr);
     for (j=0;j<m;j++) {
-      ierr = PetscSNPrintf(str,50,"%s_%d",((PetscObject)bv)->name,(int)j);CHKERRQ(ierr);
-      ierr = PetscObjectSetName((PetscObject)newV[j],str);CHKERRQ(ierr);
+      if (ctx->mpi) {
+        ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)bv->t),bs,bv->n,PETSC_DECIDE,newarray+j*bv->n,newV+j);CHKERRQ(ierr);
+      } else {
+        ierr = VecCreateSeqWithArray(PetscObjectComm((PetscObject)bv->t),bs,bv->n,newarray+j*bv->n,newV+j);CHKERRQ(ierr);
+      }
     }
+    ierr = PetscLogObjectParents(bv,m,newV);CHKERRQ(ierr);
+    if (((PetscObject)bv)->name) {
+      for (j=0;j<m;j++) {
+        ierr = PetscSNPrintf(str,50,"%s_%d",((PetscObject)bv)->name,(int)j);CHKERRQ(ierr);
+        ierr = PetscObjectSetName((PetscObject)newV[j],str);CHKERRQ(ierr);
+      }
+    }
+    if (copy) {
+      ierr = PetscMemcpy(newarray,ctx->array,PetscMin(m,bv->m)*bv->n*sizeof(PetscScalar));CHKERRQ(ierr);
+    }
+    ierr = VecDestroyVecs(bv->m,&ctx->V);CHKERRQ(ierr);
+    ctx->V = newV;
+    ierr = PetscFree(ctx->array);CHKERRQ(ierr);
+    ctx->array = newarray;
   }
-  if (copy) {
-    ierr = PetscMemcpy(newarray,ctx->array,PetscMin(m,bv->m)*bv->n*sizeof(PetscScalar));CHKERRQ(ierr);
-  }
-  ierr = VecDestroyVecs(bv->m,&ctx->V);CHKERRQ(ierr);
-  ctx->V = newV;
-  ierr = PetscFree(ctx->array);CHKERRQ(ierr);
-  ctx->array = newarray;
   PetscFunctionReturn(0);
 }
 
@@ -304,8 +307,10 @@ PetscErrorCode BVDestroy_Contiguous(BV bv)
   BV_CONTIGUOUS  *ctx = (BV_CONTIGUOUS*)bv->data;
 
   PetscFunctionBegin;
-  ierr = VecDestroyVecs(bv->nc+bv->m,&ctx->V);CHKERRQ(ierr);
-  ierr = PetscFree(ctx->array);CHKERRQ(ierr);
+  if (!bv->issplit) {
+    ierr = VecDestroyVecs(bv->nc+bv->m,&ctx->V);CHKERRQ(ierr);
+    ierr = PetscFree(ctx->array);CHKERRQ(ierr);
+  }
   ierr = PetscFree(bv->data);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -314,10 +319,13 @@ PETSC_EXTERN PetscErrorCode BVCreate_Contiguous(BV bv)
 {
   PetscErrorCode ierr;
   BV_CONTIGUOUS  *ctx;
-  PetscInt       j,nloc,bs;
+  PetscInt       j,nloc,bs,lsplit;
   PetscBool      seq;
   PetscScalar    *aa;
   char           str[50];
+  PetscScalar    *array;
+  BV             parent;
+  Vec            *Vpar;
 
   PetscFunctionBegin;
   ierr = PetscNewLog(bv,&ctx);CHKERRQ(ierr);
@@ -331,17 +339,28 @@ PETSC_EXTERN PetscErrorCode BVCreate_Contiguous(BV bv)
 
   ierr = VecGetLocalSize(bv->t,&nloc);CHKERRQ(ierr);
   ierr = VecGetBlockSize(bv->t,&bs);CHKERRQ(ierr);
-  ierr = PetscMalloc1(bv->m*nloc,&ctx->array);CHKERRQ(ierr);
-  ierr = PetscMemzero(ctx->array,bv->m*nloc*sizeof(PetscScalar));CHKERRQ(ierr);
-  ierr = PetscMalloc1(bv->m,&ctx->V);CHKERRQ(ierr);
-  for (j=0;j<bv->m;j++) {
-    if (ctx->mpi) {
-      ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)bv->t),bs,nloc,PETSC_DECIDE,ctx->array+j*nloc,ctx->V+j);CHKERRQ(ierr);
-    } else {
-      ierr = VecCreateSeqWithArray(PetscObjectComm((PetscObject)bv->t),bs,nloc,ctx->array+j*nloc,ctx->V+j);CHKERRQ(ierr);
+
+  if (bv->issplit) {
+    /* split BV: share memory and Vecs of the parent BV */
+    parent = bv->splitparent;
+    lsplit = parent->lsplit;
+    Vpar   = ((BV_CONTIGUOUS*)parent->data)->V;
+    ctx->V = (bv->issplit==1)? Vpar: Vpar+lsplit;
+    array  = ((BV_CONTIGUOUS*)parent->data)->array;
+    ctx->array = (bv->issplit==1)? array: array+lsplit*nloc;
+  } else {
+    /* regular BV: allocate memory and Vecs for the BV entries */
+    ierr = PetscCalloc1(bv->m*nloc,&ctx->array);CHKERRQ(ierr);
+    ierr = PetscMalloc1(bv->m,&ctx->V);CHKERRQ(ierr);
+    for (j=0;j<bv->m;j++) {
+      if (ctx->mpi) {
+        ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)bv->t),bs,nloc,PETSC_DECIDE,ctx->array+j*nloc,ctx->V+j);CHKERRQ(ierr);
+      } else {
+        ierr = VecCreateSeqWithArray(PetscObjectComm((PetscObject)bv->t),bs,nloc,ctx->array+j*nloc,ctx->V+j);CHKERRQ(ierr);
+      }
     }
+    ierr = PetscLogObjectParents(bv,bv->m,ctx->V);CHKERRQ(ierr);
   }
-  ierr = PetscLogObjectParents(bv,bv->m,ctx->V);CHKERRQ(ierr);
   if (((PetscObject)bv)->name) {
     for (j=0;j<bv->m;j++) {
       ierr = PetscSNPrintf(str,50,"%s_%d",((PetscObject)bv)->name,(int)j);CHKERRQ(ierr);
@@ -368,6 +387,7 @@ PETSC_EXTERN PetscErrorCode BVCreate_Contiguous(BV bv)
   bv->ops->norm_local       = BVNorm_Local_Contiguous;
   bv->ops->matmult          = BVMatMult_Contiguous;
   bv->ops->copy             = BVCopy_Contiguous;
+  bv->ops->copycolumn       = BVCopyColumn_Contiguous;
   bv->ops->resize           = BVResize_Contiguous;
   bv->ops->getcolumn        = BVGetColumn_Contiguous;
   bv->ops->getarray         = BVGetArray_Contiguous;
