@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -207,36 +207,22 @@ PetscErrorCode BVMatMult_Svec(BV V,Mat A,BV W)
 {
   PetscErrorCode ierr;
   BV_SVEC        *v = (BV_SVEC*)V->data,*w = (BV_SVEC*)W->data;
-  PetscScalar    *pv,*pw,*pb,*pc;
-  PetscInt       j,m;
+  PetscScalar    *pv,*pw;
+  PetscInt       j;
   PetscBool      flg;
+  Mat            Vmat,Wmat;
 
   PetscFunctionBegin;
-  ierr = VecGetArray(v->v,&pv);CHKERRQ(ierr);
-  ierr = VecGetArray(w->v,&pw);CHKERRQ(ierr);
   ierr = MatHasOperation(A,MATOP_MAT_MULT,&flg);CHKERRQ(ierr);
   if (V->vmm && flg) {
-    m = V->k-V->l;
-    if (V->vmm==BV_MATMULT_MAT_SAVE) {
-      ierr = BV_AllocateMatMult(V,A,m);CHKERRQ(ierr);
-      ierr = MatDenseGetArray(V->B,&pb);CHKERRQ(ierr);
-      ierr = PetscMemcpy(pb,pv+(V->nc+V->l)*V->n,m*V->n*sizeof(PetscScalar));CHKERRQ(ierr);
-      ierr = MatDenseRestoreArray(V->B,&pb);CHKERRQ(ierr);
-    } else {  /* BV_MATMULT_MAT */
-      ierr = MatCreateDense(PetscObjectComm((PetscObject)V),V->n,PETSC_DECIDE,V->N,m,pv+(V->nc+V->l)*V->n,&V->B);CHKERRQ(ierr);
-    }
-    if (!V->C) {
-      ierr = MatMatMultSymbolic(A,V->B,PETSC_DEFAULT,&V->C);CHKERRQ(ierr);
-    }
-    ierr = MatMatMultNumeric(A,V->B,V->C);CHKERRQ(ierr);
-    ierr = MatDenseGetArray(V->C,&pc);CHKERRQ(ierr);
-    ierr = PetscMemcpy(pw+(W->nc+W->l)*W->n,pc,m*V->n*sizeof(PetscScalar));CHKERRQ(ierr);
-    ierr = MatDenseRestoreArray(V->C,&pc);CHKERRQ(ierr);
-    if (V->vmm==BV_MATMULT_MAT) {
-      ierr = MatDestroy(&V->B);CHKERRQ(ierr);
-      ierr = MatDestroy(&V->C);CHKERRQ(ierr);
-    }
+    ierr = BVGetMat(V,&Vmat);CHKERRQ(ierr);
+    ierr = BVGetMat(W,&Wmat);CHKERRQ(ierr);
+    ierr = MatMatMult(A,Vmat,MAT_REUSE_MATRIX,PETSC_DEFAULT,&Wmat);CHKERRQ(ierr);
+    ierr = BVRestoreMat(V,&Vmat);CHKERRQ(ierr);
+    ierr = BVRestoreMat(W,&Wmat);CHKERRQ(ierr);
   } else {
+    ierr = VecGetArray(v->v,&pv);CHKERRQ(ierr);
+    ierr = VecGetArray(w->v,&pw);CHKERRQ(ierr);
     for (j=0;j<V->k-V->l;j++) {
       ierr = VecPlaceArray(V->cv[1],pv+(V->nc+V->l+j)*V->n);CHKERRQ(ierr);
       ierr = VecPlaceArray(W->cv[1],pw+(W->nc+W->l+j)*W->n);CHKERRQ(ierr);
@@ -244,9 +230,9 @@ PetscErrorCode BVMatMult_Svec(BV V,Mat A,BV W)
       ierr = VecResetArray(V->cv[1]);CHKERRQ(ierr);
       ierr = VecResetArray(W->cv[1]);CHKERRQ(ierr);
     }
+    ierr = VecRestoreArray(v->v,&pv);CHKERRQ(ierr);
+    ierr = VecRestoreArray(w->v,&pw);CHKERRQ(ierr);
   }
-  ierr = VecRestoreArray(v->v,&pv);CHKERRQ(ierr);
-  ierr = VecRestoreArray(w->v,&pw);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -267,35 +253,60 @@ PetscErrorCode BVCopy_Svec(BV V,BV W)
   PetscFunctionReturn(0);
 }
 
-PetscErrorCode BVResize_Svec(BV bv,PetscInt m,PetscBool copy)
+PetscErrorCode BVCopyColumn_Svec(BV V,PetscInt j,PetscInt i)
 {
   PetscErrorCode ierr;
-  BV_SVEC        *ctx = (BV_SVEC*)bv->data;
-  PetscScalar    *pv,*pnew;
-  PetscInt       bs;
-  Vec            vnew;
-  char           str[50];
+  BV_SVEC        *v = (BV_SVEC*)V->data;
+  PetscScalar    *pv;
 
   PetscFunctionBegin;
-  ierr = VecGetBlockSize(bv->t,&bs);CHKERRQ(ierr);
-  ierr = VecCreate(PetscObjectComm((PetscObject)bv->t),&vnew);CHKERRQ(ierr);
-  ierr = VecSetType(vnew,((PetscObject)bv->t)->type_name);CHKERRQ(ierr);
-  ierr = VecSetSizes(vnew,m*bv->n,PETSC_DECIDE);CHKERRQ(ierr);
-  ierr = VecSetBlockSize(vnew,bs);CHKERRQ(ierr);
-  ierr = PetscLogObjectParent((PetscObject)bv,(PetscObject)vnew);CHKERRQ(ierr);
-  if (((PetscObject)bv)->name) {
-    ierr = PetscSNPrintf(str,50,"%s_0",((PetscObject)bv)->name);CHKERRQ(ierr);
-    ierr = PetscObjectSetName((PetscObject)vnew,str);CHKERRQ(ierr);
+  ierr = VecGetArray(v->v,&pv);CHKERRQ(ierr);
+  ierr = PetscMemcpy(pv+(V->nc+i)*V->n,pv+(V->nc+j)*V->n,V->n*sizeof(PetscScalar));CHKERRQ(ierr);
+  ierr = VecRestoreArray(v->v,&pv);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode BVResize_Svec(BV bv,PetscInt m,PetscBool copy)
+{
+  PetscErrorCode    ierr;
+  BV_SVEC           *ctx = (BV_SVEC*)bv->data;
+  PetscScalar       *pnew;
+  const PetscScalar *pv;
+  PetscInt          bs,lsplit;
+  Vec               vnew,vpar;
+  char              str[50];
+  BV                parent;
+
+  PetscFunctionBegin;
+  if (bv->issplit==2) {
+    parent = bv->splitparent;
+    lsplit = parent->lsplit;
+    vpar = ((BV_SVEC*)parent->data)->v;
+    ierr = VecGetArrayRead(vpar,&pv);CHKERRQ(ierr);
+    ierr = VecResetArray(ctx->v);CHKERRQ(ierr);
+    ierr = VecPlaceArray(ctx->v,pv+lsplit*bv->n);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(vpar,&pv);CHKERRQ(ierr);
+  } else if (!bv->issplit) {
+    ierr = VecGetBlockSize(bv->t,&bs);CHKERRQ(ierr);
+    ierr = VecCreate(PetscObjectComm((PetscObject)bv->t),&vnew);CHKERRQ(ierr);
+    ierr = VecSetType(vnew,((PetscObject)bv->t)->type_name);CHKERRQ(ierr);
+    ierr = VecSetSizes(vnew,m*bv->n,PETSC_DECIDE);CHKERRQ(ierr);
+    ierr = VecSetBlockSize(vnew,bs);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent((PetscObject)bv,(PetscObject)vnew);CHKERRQ(ierr);
+    if (((PetscObject)bv)->name) {
+      ierr = PetscSNPrintf(str,50,"%s_0",((PetscObject)bv)->name);CHKERRQ(ierr);
+      ierr = PetscObjectSetName((PetscObject)vnew,str);CHKERRQ(ierr);
+    }
+    if (copy) {
+      ierr = VecGetArrayRead(ctx->v,&pv);CHKERRQ(ierr);
+      ierr = VecGetArray(vnew,&pnew);CHKERRQ(ierr);
+      ierr = PetscMemcpy(pnew,pv,PetscMin(m,bv->m)*bv->n*sizeof(PetscScalar));CHKERRQ(ierr);
+      ierr = VecRestoreArrayRead(ctx->v,&pv);CHKERRQ(ierr);
+      ierr = VecRestoreArray(vnew,&pnew);CHKERRQ(ierr);
+    }
+    ierr = VecDestroy(&ctx->v);CHKERRQ(ierr);
+    ctx->v = vnew;
   }
-  if (copy) {
-    ierr = VecGetArray(ctx->v,&pv);CHKERRQ(ierr);
-    ierr = VecGetArray(vnew,&pnew);CHKERRQ(ierr);
-    ierr = PetscMemcpy(pnew,pv,PetscMin(m,bv->m)*bv->n*sizeof(PetscScalar));CHKERRQ(ierr);
-    ierr = VecRestoreArray(ctx->v,&pv);CHKERRQ(ierr);
-    ierr = VecRestoreArray(vnew,&pnew);CHKERRQ(ierr);
-  }
-  ierr = VecDestroy(&ctx->v);CHKERRQ(ierr);
-  ctx->v = vnew;
   PetscFunctionReturn(0);
 }
 
@@ -375,10 +386,11 @@ PetscErrorCode BVView_Svec(BV bv,PetscViewer viewer)
   const char        *bvname,*name;
 
   PetscFunctionBegin;
-  ierr = VecView(ctx->v,viewer);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
     ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
+    if (format == PETSC_VIEWER_ASCII_INFO || format == PETSC_VIEWER_ASCII_INFO_DETAIL) PetscFunctionReturn(0);
+    ierr = VecView(ctx->v,viewer);CHKERRQ(ierr);
     if (format == PETSC_VIEWER_ASCII_MATLAB) {
       ierr = PetscObjectGetName((PetscObject)bv,&bvname);CHKERRQ(ierr);
       ierr = PetscObjectGetName((PetscObject)ctx->v,&name);CHKERRQ(ierr);
@@ -387,6 +399,8 @@ PetscErrorCode BVView_Svec(BV bv,PetscViewer viewer)
         ierr = PetscViewerASCIIPrintf(viewer,"%s=%s(:,%D:end);\n",bvname,bvname,bv->nc+1);CHKERRQ(ierr);
       }
     }
+  } else {
+    ierr = VecView(ctx->v,viewer);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -407,12 +421,18 @@ PetscErrorCode BVDestroy_Svec(BV bv)
 
 PETSC_EXTERN PetscErrorCode BVCreate_Svec(BV bv)
 {
-  PetscErrorCode ierr;
-  BV_SVEC        *ctx;
-  PetscInt       nloc,N,bs,tglobal,tlocal;
-  PetscBool      seq;
-  PetscScalar    *aa,*vv;
-  char           str[50];
+  PetscErrorCode    ierr;
+  BV_SVEC           *ctx;
+  PetscInt          nloc,N,bs,tglobal,tlocal,lsplit;
+  PetscBool         seq;
+  PetscScalar       *aa,*vv;
+  const PetscScalar *array,*ptr;
+  char              str[50];
+  BV                parent;
+  Vec               vpar;
+#if defined(PETSC_HAVE_VECCUDA)
+  PetscScalar       *gpuarray,*gptr;
+#endif
 
   PetscFunctionBegin;
   ierr = PetscNewLog(bv,&ctx);CHKERRQ(ierr);
@@ -431,10 +451,41 @@ PETSC_EXTERN PetscErrorCode BVCreate_Svec(BV bv)
   tglobal = bv->m*N;
   if (tglobal<0) SETERRQ2(PetscObjectComm((PetscObject)bv),1,"The product %D times %D overflows the size of PetscInt; consider reducing the number of columns, or use BVVECS instead",bv->m,N);
 
-  ierr = VecCreate(PetscObjectComm((PetscObject)bv->t),&ctx->v);CHKERRQ(ierr);
-  ierr = VecSetType(ctx->v,((PetscObject)bv->t)->type_name);CHKERRQ(ierr);
-  ierr = VecSetSizes(ctx->v,tlocal,tglobal);CHKERRQ(ierr);
-  ierr = VecSetBlockSize(ctx->v,bs);CHKERRQ(ierr);
+  if (bv->issplit) {
+    /* split BV: create Vec sharing the memory of the parent BV */
+    parent = bv->splitparent;
+    lsplit = parent->lsplit;
+    vpar = ((BV_SVEC*)parent->data)->v;
+    if (bv->cuda) {
+#if defined(PETSC_HAVE_VECCUDA)
+      ierr = VecCUDAGetArrayReadWrite(vpar,&gpuarray);CHKERRQ(ierr);
+      gptr = (bv->issplit==1)? gpuarray: gpuarray+lsplit*nloc;
+      ierr = VecCUDARestoreArrayReadWrite(vpar,&gpuarray);CHKERRQ(ierr);
+      if (ctx->mpi) {
+        ierr = VecCreateMPICUDAWithArray(PetscObjectComm((PetscObject)bv->t),bs,tlocal,bv->m*N,NULL,&ctx->v);CHKERRQ(ierr);
+      } else {
+        ierr = VecCreateSeqCUDAWithArray(PetscObjectComm((PetscObject)bv->t),bs,tlocal,NULL,&ctx->v);CHKERRQ(ierr);
+      }
+      ierr = VecCUDAPlaceArray(ctx->v,gptr);CHKERRQ(ierr);
+#endif
+    } else {
+      ierr = VecGetArrayRead(vpar,&array);CHKERRQ(ierr);
+      ptr = (bv->issplit==1)? array: array+lsplit*nloc;
+      ierr = VecRestoreArrayRead(vpar,&array);CHKERRQ(ierr);
+      if (ctx->mpi) {
+        ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)bv->t),bs,tlocal,bv->m*N,NULL,&ctx->v);CHKERRQ(ierr);
+      } else {
+        ierr = VecCreateSeqWithArray(PetscObjectComm((PetscObject)bv->t),bs,tlocal,NULL,&ctx->v);CHKERRQ(ierr);
+      }
+      ierr = VecPlaceArray(ctx->v,ptr);CHKERRQ(ierr);
+    }
+  } else {
+    /* regular BV: create Vec to store the BV entries */
+    ierr = VecCreate(PetscObjectComm((PetscObject)bv->t),&ctx->v);CHKERRQ(ierr);
+    ierr = VecSetType(ctx->v,((PetscObject)bv->t)->type_name);CHKERRQ(ierr);
+    ierr = VecSetSizes(ctx->v,tlocal,tglobal);CHKERRQ(ierr);
+    ierr = VecSetBlockSize(ctx->v,bs);CHKERRQ(ierr);
+  }
   ierr = PetscLogObjectParent((PetscObject)bv,(PetscObject)ctx->v);CHKERRQ(ierr);
   if (((PetscObject)bv)->name) {
     ierr = PetscSNPrintf(str,50,"%s_0",((PetscObject)bv)->name);CHKERRQ(ierr);
@@ -452,8 +503,7 @@ PETSC_EXTERN PetscErrorCode BVCreate_Svec(BV bv)
 
   ierr = VecDuplicateEmpty(bv->t,&bv->cv[0]);CHKERRQ(ierr);
   ierr = VecDuplicateEmpty(bv->t,&bv->cv[1]);CHKERRQ(ierr);
-  ierr = VecSetType(bv->cv[0],((PetscObject)bv->t)->type_name);CHKERRQ(ierr);
-  ierr = VecSetType(bv->cv[1],((PetscObject)bv->t)->type_name);CHKERRQ(ierr);
+
   if (bv->cuda) {
 #if defined(PETSC_HAVE_VECCUDA)
     bv->ops->mult             = BVMult_Svec_CUDA;
@@ -466,9 +516,11 @@ PETSC_EXTERN PetscErrorCode BVCreate_Svec(BV bv)
     bv->ops->scale            = BVScale_Svec_CUDA;
     bv->ops->matmult          = BVMatMult_Svec_CUDA;
     bv->ops->copy             = BVCopy_Svec_CUDA;
+    bv->ops->copycolumn       = BVCopyColumn_Svec_CUDA;
     bv->ops->resize           = BVResize_Svec_CUDA;
     bv->ops->getcolumn        = BVGetColumn_Svec_CUDA;
     bv->ops->restorecolumn    = BVRestoreColumn_Svec_CUDA;
+    bv->ops->restoresplit     = BVRestoreSplit_Svec_CUDA;
 #endif
   } else {
     bv->ops->mult             = BVMult_Svec;
@@ -481,6 +533,7 @@ PETSC_EXTERN PetscErrorCode BVCreate_Svec(BV bv)
     bv->ops->scale            = BVScale_Svec;
     bv->ops->matmult          = BVMatMult_Svec;
     bv->ops->copy             = BVCopy_Svec;
+    bv->ops->copycolumn       = BVCopyColumn_Svec;
     bv->ops->resize           = BVResize_Svec;
     bv->ops->getcolumn        = BVGetColumn_Svec;
     bv->ops->restorecolumn    = BVRestoreColumn_Svec;
