@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -43,6 +43,8 @@ PetscErrorCode BVSetType(BV bv,BVType type)
 
   ierr = PetscObjectTypeCompare((PetscObject)bv,type,&match);CHKERRQ(ierr);
   if (match) PetscFunctionReturn(0);
+  ierr = PetscStrcmp(type,BVTENSOR,&match);CHKERRQ(ierr);
+  if (match) SETERRQ(PetscObjectComm((PetscObject)bv),1,"Use BVCreateTensor() to create a BV of type tensor");
 
   ierr =  PetscFunctionListFind(BVList,type,&r);CHKERRQ(ierr);
   if (!r) SETERRQ1(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_UNKNOWN_TYPE,"Unable to find requested BV type %s",type);
@@ -341,7 +343,7 @@ PetscErrorCode BVResize(BV bv,PetscInt m,PetscBool copy)
   PetscValidLogicalCollectiveBool(bv,copy,3);
   PetscValidType(bv,1);
   if (m <= 0) SETERRQ1(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_INCOMP,"Number of columns %D must be positive",m);
-  if (bv->nc) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_WRONGSTATE,"Cannot resize a BV with constraints");
+  if (bv->nc && !bv->issplit) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_WRONGSTATE,"Cannot resize a BV with constraints");
   if (bv->m == m) PetscFunctionReturn(0);
 
   ierr = PetscLogEventBegin(BV_Create,bv,0,0,0);CHKERRQ(ierr);
@@ -501,10 +503,6 @@ PetscErrorCode BVSetMatrix(BV bv,Mat B,PetscBool indef)
   if (B) PetscObjectReference((PetscObject)B);
   bv->matrix = B;
   bv->indef  = indef;
-  if (B && !bv->Bx) {
-    ierr = MatCreateVecs(B,&bv->Bx,NULL);CHKERRQ(ierr);
-    ierr = PetscLogObjectParent((PetscObject)bv,(PetscObject)bv->Bx);CHKERRQ(ierr);
-  }
   ierr = PetscObjectStateIncrease((PetscObject)bv);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -613,44 +611,6 @@ PetscErrorCode BVApplyMatrixBV(BV X,BV Y)
   PetscFunctionReturn(0);
 }
 
-/*@
-   BVGetCachedBV - Returns a BV object stored internally that holds the
-   result of B*X after a call to BVApplyMatrixBV().
-
-   Not collective
-
-   Input Parameter:
-.  bv    - the basis vectors context
-
-   Output Parameter:
-.  cached - the cached BV
-
-   Note:
-   The cached BV is created if not available previously.
-
-   Level: developer
-
-.seealso: BVApplyMatrixBV()
-@*/
-PetscErrorCode BVGetCachedBV(BV bv,BV *cached)
-{
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(bv,BV_CLASSID,1);
-  PetscValidPointer(cached,2);
-  BVCheckSizes(bv,1);
-  if (!bv->cached) {
-    ierr = BVCreate(PetscObjectComm((PetscObject)bv),&bv->cached);CHKERRQ(ierr);
-    ierr = BVSetSizesFromVec(bv->cached,bv->t,bv->m);CHKERRQ(ierr);
-    ierr = BVSetType(bv->cached,((PetscObject)bv)->type_name);CHKERRQ(ierr);
-    ierr = BVSetOrthogonalization(bv->cached,bv->orthog_type,bv->orthog_ref,bv->orthog_eta,bv->orthog_block);CHKERRQ(ierr);
-    ierr = PetscLogObjectParent((PetscObject)bv,(PetscObject)bv->cached);CHKERRQ(ierr);
-  }
-  *cached = bv->cached;
-  PetscFunctionReturn(0);
-}
-
 /*@
    BVSetSignature - Sets the signature matrix to be used in orthogonalization.
 
@@ -1032,6 +992,8 @@ PetscErrorCode BVSetOrthogonalization(BV bv,BVOrthogType type,BVOrthogRefineType
     case BV_ORTHOG_BLOCK_GS:
     case BV_ORTHOG_BLOCK_CHOL:
     case BV_ORTHOG_BLOCK_TSQR:
+    case BV_ORTHOG_BLOCK_TSQRCHOL:
+    case BV_ORTHOG_BLOCK_SVQB:
       bv->orthog_block = block;
       break;
     default:
@@ -1079,14 +1041,15 @@ PetscErrorCode BVGetOrthogonalization(BV bv,BVOrthogType *type,BVOrthogRefineTyp
 -  method - the method for the BVMatMult() operation
 
    Options Database Keys:
-.  -bv_matmult <meth> - choose one of the methods: vecs, mat, mat_save
+.  -bv_matmult <meth> - choose one of the methods: vecs, mat
 
-   Note:
-   Allowed values are:
+   Notes:
+   Allowed values are
 +  BV_MATMULT_VECS - perform a matrix-vector multiply per each column
 .  BV_MATMULT_MAT - carry out a MatMatMult() product with a dense matrix
--  BV_MATMULT_MAT_SAVE - call MatMatMult() and keep auxiliary matrices
-   The default is BV_MATMULT_MAT.
+-  BV_MATMULT_MAT_SAVE - this case is deprecated
+
+   The default is BV_MATMULT_MAT except in the case of BVVECS.
 
    Level: advanced
 
@@ -1094,15 +1057,20 @@ PetscErrorCode BVGetOrthogonalization(BV bv,BVOrthogType *type,BVOrthogRefineTyp
 @*/
 PetscErrorCode BVSetMatMultMethod(BV bv,BVMatMultType method)
 {
+  PetscErrorCode ierr;
+
   PetscFunctionBegin;
   PetscValidHeaderSpecific(bv,BV_CLASSID,1);
   PetscValidLogicalCollectiveEnum(bv,method,2);
   switch (method) {
     case BV_MATMULT_VECS:
     case BV_MATMULT_MAT:
-    case BV_MATMULT_MAT_SAVE:
       bv->vmm = method;
       break;
+    case BV_MATMULT_MAT_SAVE:
+      ierr = PetscInfo(bv,"BV_MATMULT_MAT_SAVE is deprecated, using BV_MATMULT_MAT\n");CHKERRQ(ierr);
+      bv->vmm = BV_MATMULT_MAT;
+      break;
     default:
       SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_WRONG,"Unknown matmult method");
   }
@@ -1413,12 +1381,14 @@ PetscErrorCode BVCreateVec(BV bv,Vec *v)
    Output Parameter:
 .  A  - the new matrix
 
-   Note:
+   Notes:
    The user is responsible of destroying the returned matrix.
 
+   The matrix contains all columns of the BV, not just the active columns.
+
    Level: intermediate
 
-.seealso: BVCreateFromMat(), BVCreateVec()
+.seealso: BVCreateFromMat(), BVCreateVec(), BVGetMat()
 @*/
 PetscErrorCode BVCreateMat(BV bv,Mat *A)
 {
@@ -1442,20 +1412,122 @@ PetscErrorCode BVCreateMat(BV bv,Mat *A)
   PetscFunctionReturn(0);
 }
 
-PETSC_STATIC_INLINE PetscErrorCode BVDuplicate_Private(BV V,PetscInt m,BV *W)
+/*@
+   BVGetMat - Returns a Mat object of dense type that shares the memory of
+   the BV object.
+
+   Collective on BV
+
+   Input Parameter:
+.  bv - the basis vectors context
+
+   Output Parameter:
+.  A  - the matrix
+
+   Notes:
+   The returned matrix contains only the active columns. If the content of
+   the Mat is modified, these changes are also done in the BV object. The
+   user must call BVRestoreMat() when no longer needed.
+
+   This operation implies a call to BVGetArray(), which may result in data
+   copies.
+
+   Level: advanced
+
+.seealso: BVRestoreMat(), BVCreateMat(), BVGetArray()
+@*/
+PetscErrorCode BVGetMat(BV bv,Mat *A)
+{
+  PetscErrorCode ierr;
+  PetscScalar    *vv,*aa;
+  PetscBool      create=PETSC_FALSE;
+  PetscInt       m,cols;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(bv,BV_CLASSID,1);
+  BVCheckSizes(bv,1);
+  PetscValidPointer(A,2);
+  m = bv->k-bv->l;
+  if (!bv->Aget) create=PETSC_TRUE;
+  else {
+    ierr = MatDenseGetArray(bv->Aget,&aa);CHKERRQ(ierr);
+    if (aa) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_WRONGSTATE,"BVGetMat already called on this BV");
+    ierr = MatGetSize(bv->Aget,NULL,&cols);CHKERRQ(ierr);
+    if (cols!=m) {
+      ierr = MatDestroy(&bv->Aget);CHKERRQ(ierr);
+      create=PETSC_TRUE;
+    }
+  }
+  ierr = BVGetArray(bv,&vv);CHKERRQ(ierr);
+  if (create) {
+    ierr = MatCreateDense(PetscObjectComm((PetscObject)bv),bv->n,PETSC_DECIDE,bv->N,m,vv,&bv->Aget);CHKERRQ(ierr); /* pass a pointer to avoid allocation of storage */
+    ierr = MatDensePlaceArray(bv->Aget,NULL);CHKERRQ(ierr);  /* replace with a null pointer, the value after BVRestoreMat */
+    ierr = PetscLogObjectParent((PetscObject)bv,(PetscObject)bv->Aget);CHKERRQ(ierr);
+    ierr = MatAssemblyBegin(bv->Aget,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
+    ierr = MatAssemblyEnd(bv->Aget,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
+  }
+  ierr = MatDensePlaceArray(bv->Aget,vv+(bv->nc+bv->l)*bv->n);CHKERRQ(ierr);  /* set the actual pointer */
+  *A = bv->Aget;
+  PetscFunctionReturn(0);
+}
+
+/*@
+   BVRestoreMat - Restores the Mat obtained with BVGetMat().
+
+   Collective on BV
+
+   Input Parameters:
++  bv - the basis vectors context
+-  A  - the fetched matrix
+
+   Note:
+   A call to this function must match a previous call of BVGetMat().
+   The effect is that the contents of the Mat are copied back to the
+   BV internal data structures.
+
+   Level: advanced
+
+.seealso: BVGetMat(), BVRestoreArray()
+@*/
+PetscErrorCode BVRestoreMat(BV bv,Mat *A)
+{
+  PetscErrorCode ierr;
+  PetscScalar    *vv,*aa;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(bv,BV_CLASSID,1);
+  BVCheckSizes(bv,1);
+  PetscValidPointer(A,2);
+  if (!bv->Aget) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_WRONGSTATE,"BVRestoreMat must match a previous call to BVGetMat");
+  if (bv->Aget!=*A) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_WRONGSTATE,"Mat argument is not the same as the one obtained with BVGetMat");
+  ierr = MatDenseGetArray(bv->Aget,&aa);CHKERRQ(ierr);
+  vv = aa-(bv->nc+bv->l)*bv->n;
+  ierr = MatDenseResetArray(bv->Aget);CHKERRQ(ierr);
+  ierr = BVRestoreArray(bv,&vv);CHKERRQ(ierr);
+  *A = NULL;
+  PetscFunctionReturn(0);
+}
+
+/*
+   Copy all user-provided attributes of V to another BV object W
+ */
+PETSC_STATIC_INLINE PetscErrorCode BVDuplicate_Private(BV V,BV W)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = BVCreate(PetscObjectComm((PetscObject)V),W);CHKERRQ(ierr);
-  ierr = BVSetSizesFromVec(*W,V->t,m);CHKERRQ(ierr);
-  ierr = BVSetType(*W,((PetscObject)V)->type_name);CHKERRQ(ierr);
-  ierr = BVSetMatrix(*W,V->matrix,V->indef);CHKERRQ(ierr);
-  ierr = BVSetOrthogonalization(*W,V->orthog_type,V->orthog_ref,V->orthog_eta,V->orthog_block);CHKERRQ(ierr);
-  (*W)->vmm     = V->vmm;
-  (*W)->rrandom = V->rrandom;
+  ierr = BVSetType(W,((PetscObject)V)->type_name);CHKERRQ(ierr);
+  W->orthog_type  = V->orthog_type;
+  W->orthog_ref   = V->orthog_ref;
+  W->orthog_eta   = V->orthog_eta;
+  W->orthog_block = V->orthog_block;
+  if (V->matrix) PetscObjectReference((PetscObject)V->matrix);
+  W->matrix       = V->matrix;
+  W->indef        = V->indef;
+  W->vmm          = V->vmm;
+  W->rrandom      = V->rrandom;
   if (V->ops->duplicate) { ierr = (*V->ops->duplicate)(V,W);CHKERRQ(ierr); }
-  ierr = PetscObjectStateIncrease((PetscObject)*W);CHKERRQ(ierr);
+  ierr = PetscObjectStateIncrease((PetscObject)W);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -1492,7 +1564,9 @@ PetscErrorCode BVDuplicate(BV V,BV *W)
   PetscValidType(V,1);
   BVCheckSizes(V,1);
   PetscValidPointer(W,2);
-  ierr = BVDuplicate_Private(V,V->m,W);CHKERRQ(ierr);
+  ierr = BVCreate(PetscObjectComm((PetscObject)V),W);CHKERRQ(ierr);
+  ierr = BVSetSizesFromVec(*W,V->t,V->m);CHKERRQ(ierr);
+  ierr = BVDuplicate_Private(V,*W);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -1527,7 +1601,46 @@ PetscErrorCode BVDuplicateResize(BV V,PetscInt m,BV *W)
   BVCheckSizes(V,1);
   PetscValidLogicalCollectiveInt(V,m,2);
   PetscValidPointer(W,3);
-  ierr = BVDuplicate_Private(V,m,W);CHKERRQ(ierr);
+  ierr = BVCreate(PetscObjectComm((PetscObject)V),W);CHKERRQ(ierr);
+  ierr = BVSetSizesFromVec(*W,V->t,m);CHKERRQ(ierr);
+  ierr = BVDuplicate_Private(V,*W);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+/*@
+   BVGetCachedBV - Returns a BV object stored internally that holds the
+   result of B*X after a call to BVApplyMatrixBV().
+
+   Not collective
+
+   Input Parameter:
+.  bv    - the basis vectors context
+
+   Output Parameter:
+.  cached - the cached BV
+
+   Note:
+   The cached BV is created if not available previously.
+
+   Level: developer
+
+.seealso: BVApplyMatrixBV()
+@*/
+PetscErrorCode BVGetCachedBV(BV bv,BV *cached)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(bv,BV_CLASSID,1);
+  PetscValidPointer(cached,2);
+  BVCheckSizes(bv,1);
+  if (!bv->cached) {
+    ierr = BVCreate(PetscObjectComm((PetscObject)bv),&bv->cached);CHKERRQ(ierr);
+    ierr = BVSetSizesFromVec(bv->cached,bv->t,bv->m);CHKERRQ(ierr);
+    ierr = BVDuplicate_Private(bv,bv->cached);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent((PetscObject)bv,(PetscObject)bv->cached);CHKERRQ(ierr);
+  }
+  *cached = bv->cached;
   PetscFunctionReturn(0);
 }
 
@@ -1665,13 +1778,140 @@ PetscErrorCode BVCopyColumn(BV V,PetscInt j,PetscInt i)
     omega[i] = omega[j];
     ierr = VecRestoreArray(V->omega,&omega);CHKERRQ(ierr);
   }
-  ierr = BVGetColumn(V,j,&z);CHKERRQ(ierr);
-  ierr = BVGetColumn(V,i,&w);CHKERRQ(ierr);
-  ierr = VecCopy(z,w);CHKERRQ(ierr);
-  ierr = BVRestoreColumn(V,j,&z);CHKERRQ(ierr);
-  ierr = BVRestoreColumn(V,i,&w);CHKERRQ(ierr);
+  if (V->ops->copycolumn) {
+    ierr = (*V->ops->copycolumn)(V,j,i);CHKERRQ(ierr);
+  } else {
+    ierr = BVGetColumn(V,j,&z);CHKERRQ(ierr);
+    ierr = BVGetColumn(V,i,&w);CHKERRQ(ierr);
+    ierr = VecCopy(z,w);CHKERRQ(ierr);
+    ierr = BVRestoreColumn(V,j,&z);CHKERRQ(ierr);
+    ierr = BVRestoreColumn(V,i,&w);CHKERRQ(ierr);
+  }
   ierr = PetscLogEventEnd(BV_Copy,V,0,0,0);CHKERRQ(ierr);
   ierr = PetscObjectStateIncrease((PetscObject)V);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
+static PetscErrorCode BVGetSplit_Private(BV bv,PetscBool left,BV *split)
+{
+  PetscErrorCode ierr;
+  PetscInt       ncols;
+
+  PetscFunctionBegin;
+  ncols = left? bv->nc+bv->l: bv->m-bv->l;
+  if (!*split) {
+    ierr = BVCreate(PetscObjectComm((PetscObject)bv),split);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent((PetscObject)bv,(PetscObject)*split);CHKERRQ(ierr);
+    (*split)->issplit = left? 1: 2;
+    (*split)->splitparent = bv;
+    ierr = BVSetSizesFromVec(*split,bv->t,ncols);CHKERRQ(ierr);
+    ierr = BVDuplicate_Private(bv,*split);CHKERRQ(ierr);
+  } else {
+    ierr = BVResize(*split,ncols,PETSC_FALSE);CHKERRQ(ierr);
+  }
+  (*split)->l  = 0;
+  (*split)->k  = left? bv->l: bv->k-bv->l;
+  (*split)->nc = left? bv->nc: 0;
+  (*split)->m  = ncols-(*split)->nc;
+  if ((*split)->nc) {
+    (*split)->ci[0] = -(*split)->nc-1;
+    (*split)->ci[1] = -(*split)->nc-1;
+  }
+  if (left) {
+    ierr = PetscObjectStateGet((PetscObject)*split,&bv->lstate);CHKERRQ(ierr);
+  } else {
+    ierr = PetscObjectStateGet((PetscObject)*split,&bv->rstate);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
+/*@
+   BVGetSplit - Splits the BV object into two BV objects that share the
+   internal data, one of them containing the leading columns and the other
+   one containing the remaining columns.
+
+   Logically Collective on BV
+
+   Input Parameters:
+.  bv - the basis vectors context
+
+   Output Parameters:
++  L - left BV containing leading columns (can be NULL)
+-  R - right BV containing remaining columns (can be NULL)
+
+   Notes:
+   The columns are split in two sets. The leading columns (including the
+   constraints) are assigned to the left BV and the remaining columns
+   are assigned to the right BV. The number of leading columns, as
+   specified with BVSetActiveColumns(), must be between 1 and m-1 (to
+   guarantee that both L and R have at least one column).
+
+   The returned BV's must be seen as references (not copies) of the input
+   BV, that is, modifying them will change the entries of bv as well.
+   The returned BV's must not be destroyed. BVRestoreSplit() must be called
+   when they are no longer needed.
+
+   Pass NULL for any of the output BV's that is not needed.
+
+   Level: advanced
+
+.seealso: BVRestoreSplit(), BVSetActiveColumns(), BVSetNumConstraints()
+@*/
+PetscErrorCode BVGetSplit(BV bv,BV *L,BV *R)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(bv,BV_CLASSID,1);
+  PetscValidType(bv,1);
+  BVCheckSizes(bv,1);
+  if (!bv->l) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_WRONGSTATE,"Must indicate the number of leading columns with BVSetActiveColumns()");
+  if (bv->lsplit) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_WRONGSTATE,"Cannot get the split BV's twice before restoring them with BVRestoreSplit()");
+  bv->lsplit = bv->nc+bv->l;
+  ierr = BVGetSplit_Private(bv,PETSC_TRUE,&bv->L);CHKERRQ(ierr);
+  ierr = BVGetSplit_Private(bv,PETSC_FALSE,&bv->R);CHKERRQ(ierr);
+  if (L) *L = bv->L;
+  if (R) *R = bv->R;
+  PetscFunctionReturn(0);
+}
+
+/*@
+   BVRestoreSplit - Restore the BV objects obtained with BVGetSplit().
+
+   Logically Collective on BV
+
+   Input Parameters:
++  bv - the basis vectors context
+.  L  - left BV obtained with BVGetSplit()
+-  R  - right BV obtained with BVGetSplit()
+
+   Note:
+   The arguments must match the corresponding call to BVGetSplit().
+
+   Level: advanced
+
+.seealso: BVGetSplit()
+@*/
+PetscErrorCode BVRestoreSplit(BV bv,BV *L,BV *R)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(bv,BV_CLASSID,1);
+  PetscValidType(bv,1);
+  BVCheckSizes(bv,1);
+  if (!bv->lsplit) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_WRONGSTATE,"Must call BVGetSplit first");
+  if (L && *L!=bv->L) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_WRONG,"Argument 2 is not the same BV that was obtained with BVGetSplit");
+  if (R && *R!=bv->R) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_WRONG,"Argument 3 is not the same BV that was obtained with BVGetSplit");
+  if (L && ((*L)->ci[0]>(*L)->nc-1 || (*L)->ci[1]>(*L)->nc-1)) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_WRONGSTATE,"Argument 2 has unrestored columns, use BVRestoreColumn()");
+  if (R && ((*R)->ci[0]>(*R)->nc-1 || (*R)->ci[1]>(*R)->nc-1)) SETERRQ(PetscObjectComm((PetscObject)bv),PETSC_ERR_ARG_WRONGSTATE,"Argument 3 has unrestored columns, use BVRestoreColumn()");
+
+  if (bv->ops->restoresplit) {
+    ierr = (*bv->ops->restoresplit)(bv,L,R);CHKERRQ(ierr);
+  }
+  bv->lsplit = 0;
+  if (L) *L = NULL;
+  if (R) *R = NULL;
+  PetscFunctionReturn(0);
+}
+
