@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -55,8 +55,7 @@ PetscErrorCode DSFinalizePackage(void)
 PetscErrorCode DSInitializePackage()
 {
   char             logList[256];
-  char             *className;
-  PetscBool        opt;
+  PetscBool      opt,pkg;
   PetscErrorCode   ierr;
 
   PetscFunctionBegin;
@@ -72,21 +71,18 @@ PetscErrorCode DSInitializePackage()
   ierr = PetscLogEventRegister("DSSynchronize",DS_CLASSID,&DS_Synchronize);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("DSOther",DS_CLASSID,&DS_Other);CHKERRQ(ierr);
   /* Process info exclusions */
-  ierr = PetscOptionsGetString(NULL,NULL,"-info_exclude",logList,256,&opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,NULL,"-info_exclude",logList,sizeof(logList),&opt);CHKERRQ(ierr);
   if (opt) {
-    ierr = PetscStrstr(logList,"ds",&className);CHKERRQ(ierr);
-    if (className) {
-      ierr = PetscInfoDeactivateClass(DS_CLASSID);CHKERRQ(ierr);
-    }
+    ierr = PetscStrInList("ds",logList,',',&pkg);CHKERRQ(ierr);
+    if (pkg) { ierr = PetscInfoDeactivateClass(DS_CLASSID);CHKERRQ(ierr); }
   }
   /* Process summary exclusions */
-  ierr = PetscOptionsGetString(NULL,NULL,"-log_exclude",logList,256,&opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,NULL,"-log_exclude",logList,sizeof(logList),&opt);CHKERRQ(ierr);
   if (opt) {
-    ierr = PetscStrstr(logList,"ds",&className);CHKERRQ(ierr);
-    if (className) {
-      ierr = PetscLogEventDeactivateClass(DS_CLASSID);CHKERRQ(ierr);
-    }
+    ierr = PetscStrInList("ds",logList,',',&pkg);CHKERRQ(ierr);
+    if (pkg) { ierr = PetscLogEventDeactivateClass(DS_CLASSID);CHKERRQ(ierr); }
   }
+  /* Register package finalizer */
   ierr = PetscRegisterFinalize(DSFinalizePackage);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -756,6 +752,7 @@ PetscErrorCode DSSetFromOptions(DS ds)
 PetscErrorCode DSView(DS ds,PetscViewer viewer)
 {
   PetscBool         isascii,issvd;
+  PetscInt          tabs;
   PetscViewerFormat format;
   PetscErrorCode    ierr;
   PetscMPIInt       size;
@@ -767,6 +764,8 @@ PetscErrorCode DSView(DS ds,PetscViewer viewer)
   PetscCheckSameComm(ds,1,viewer,2);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
+    ierr = PetscViewerASCIIGetTab(viewer,&tabs);CHKERRQ(ierr);
+    ierr = PetscViewerASCIISetTab(viewer,((PetscObject)ds)->tablevel);CHKERRQ(ierr);
     ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
     ierr = PetscObjectPrintClassNamePrefixType((PetscObject)ds,viewer);CHKERRQ(ierr);
     ierr = MPI_Comm_size(PetscObjectComm((PetscObject)ds),&size);CHKERRQ(ierr);
@@ -793,6 +792,7 @@ PetscErrorCode DSView(DS ds,PetscViewer viewer)
       ierr = (*ds->ops->view)(ds,viewer);CHKERRQ(ierr);
       ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
     }
+    ierr = PetscViewerASCIISetTab(viewer,tabs);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
