@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -52,10 +52,9 @@ PetscErrorCode FNFinalizePackage(void)
 @*/
 PetscErrorCode FNInitializePackage(void)
 {
-  char             logList[256];
-  char             *className;
-  PetscBool        opt;
-  PetscErrorCode   ierr;
+  char           logList[256];
+  PetscBool      opt,pkg;
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
   if (FNPackageInitialized) PetscFunctionReturn(0);
@@ -67,21 +66,18 @@ PetscErrorCode FNInitializePackage(void)
   /* Register Events */
   ierr = PetscLogEventRegister("FNEvaluate",FN_CLASSID,&FN_Evaluate);CHKERRQ(ierr);
   /* Process info exclusions */
-  ierr = PetscOptionsGetString(NULL,NULL,"-info_exclude",logList,256,&opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,NULL,"-info_exclude",logList,sizeof(logList),&opt);CHKERRQ(ierr);
   if (opt) {
-    ierr = PetscStrstr(logList,"fn",&className);CHKERRQ(ierr);
-    if (className) {
-      ierr = PetscInfoDeactivateClass(FN_CLASSID);CHKERRQ(ierr);
-    }
+    ierr = PetscStrInList("fn",logList,',',&pkg);CHKERRQ(ierr);
+    if (pkg) { ierr = PetscInfoDeactivateClass(FN_CLASSID);CHKERRQ(ierr); }
   }
   /* Process summary exclusions */
-  ierr = PetscOptionsGetString(NULL,NULL,"-log_exclude",logList,256,&opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,NULL,"-log_exclude",logList,sizeof(logList),&opt);CHKERRQ(ierr);
   if (opt) {
-    ierr = PetscStrstr(logList,"fn",&className);CHKERRQ(ierr);
-    if (className) {
-      ierr = PetscLogEventDeactivateClass(FN_CLASSID);CHKERRQ(ierr);
-    }
+    ierr = PetscStrInList("fn",logList,',',&pkg);CHKERRQ(ierr);
+    if (pkg) { ierr = PetscLogEventDeactivateClass(FN_CLASSID);CHKERRQ(ierr); }
   }
+  /* Register package finalizer */
   ierr = PetscRegisterFinalize(FNFinalizePackage);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -947,6 +943,7 @@ PetscErrorCode FNSetFromOptions(FN fn)
 PetscErrorCode FNView(FN fn,PetscViewer viewer)
 {
   PetscBool      isascii;
+  PetscInt       tabs;
   PetscErrorCode ierr;
   PetscMPIInt    size;
 
@@ -957,6 +954,8 @@ PetscErrorCode FNView(FN fn,PetscViewer viewer)
   PetscCheckSameComm(fn,1,viewer,2);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
+    ierr = PetscViewerASCIIGetTab(viewer,&tabs);CHKERRQ(ierr);
+    ierr = PetscViewerASCIISetTab(viewer,((PetscObject)fn)->tablevel);CHKERRQ(ierr);
     ierr = PetscObjectPrintClassNamePrefixType((PetscObject)fn,viewer);CHKERRQ(ierr);
     ierr = MPI_Comm_size(PetscObjectComm((PetscObject)fn),&size);CHKERRQ(ierr);
     if (size>1) {
@@ -967,6 +966,7 @@ PetscErrorCode FNView(FN fn,PetscViewer viewer)
       ierr = (*fn->ops->view)(fn,viewer);CHKERRQ(ierr);
       ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
     }
+    ierr = PetscViewerASCIISetTab(viewer,tabs);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
