@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2018, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -48,10 +48,9 @@ PetscErrorCode RGFinalizePackage(void)
 @*/
 PetscErrorCode RGInitializePackage(void)
 {
-  char             logList[256];
-  char             *className;
-  PetscBool        opt;
-  PetscErrorCode   ierr;
+  char           logList[256];
+  PetscBool      opt,pkg;
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
   if (RGPackageInitialized) PetscFunctionReturn(0);
@@ -61,21 +60,18 @@ PetscErrorCode RGInitializePackage(void)
   /* Register Constructors */
   ierr = RGRegisterAll();CHKERRQ(ierr);
   /* Process info exclusions */
-  ierr = PetscOptionsGetString(NULL,NULL,"-info_exclude",logList,256,&opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,NULL,"-info_exclude",logList,sizeof(logList),&opt);CHKERRQ(ierr);
   if (opt) {
-    ierr = PetscStrstr(logList,"rg",&className);CHKERRQ(ierr);
-    if (className) {
-      ierr = PetscInfoDeactivateClass(RG_CLASSID);CHKERRQ(ierr);
-    }
+    ierr = PetscStrInList("rg",logList,',',&pkg);CHKERRQ(ierr);
+    if (pkg) { ierr = PetscInfoDeactivateClass(RG_CLASSID);CHKERRQ(ierr); }
   }
   /* Process summary exclusions */
-  ierr = PetscOptionsGetString(NULL,NULL,"-log_exclude",logList,256,&opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,NULL,"-log_exclude",logList,sizeof(logList),&opt);CHKERRQ(ierr);
   if (opt) {
-    ierr = PetscStrstr(logList,"rg",&className);CHKERRQ(ierr);
-    if (className) {
-      ierr = PetscLogEventDeactivateClass(RG_CLASSID);CHKERRQ(ierr);
-    }
+    ierr = PetscStrInList("rg",logList,',',&pkg);CHKERRQ(ierr);
+    if (pkg) { ierr = PetscLogEventDeactivateClass(RG_CLASSID);CHKERRQ(ierr); }
   }
+  /* Register package finalizer */
   ierr = PetscRegisterFinalize(RGFinalizePackage);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -330,7 +326,8 @@ PetscErrorCode RGSetFromOptions(RG rg)
 @*/
 PetscErrorCode RGView(RG rg,PetscViewer viewer)
 {
-  PetscBool      isascii;
+  PetscBool      isdraw,isascii;
+  PetscInt       tabs;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
@@ -338,8 +335,11 @@ PetscErrorCode RGView(RG rg,PetscViewer viewer)
   if (!viewer) viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)rg));
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   PetscCheckSameComm(rg,1,viewer,2);
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERDRAW,&isdraw);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
+    ierr = PetscViewerASCIIGetTab(viewer,&tabs);CHKERRQ(ierr);
+    ierr = PetscViewerASCIISetTab(viewer,((PetscObject)rg)->tablevel);CHKERRQ(ierr);
     ierr = PetscObjectPrintClassNamePrefixType((PetscObject)rg,viewer);CHKERRQ(ierr);
     if (rg->ops->view) {
       ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
@@ -352,6 +352,9 @@ PetscErrorCode RGView(RG rg,PetscViewer viewer)
     if (rg->sfactor!=1.0) {
       ierr = PetscViewerASCIIPrintf(viewer,"  scaling factor = %g\n",(double)rg->sfactor);CHKERRQ(ierr);
     }
+    ierr = PetscViewerASCIISetTab(viewer,tabs);CHKERRQ(ierr);
+  } else if (isdraw) {
+    if (rg->ops->view) { ierr = (*rg->ops->view)(rg,viewer);CHKERRQ(ierr); }
   }
   PetscFunctionReturn(0);
 }
